#include <conf.h>
#include <kernel.h>
#include <netdefs.h>
#include <arp.h>
#include <network.h>
#include <mq.h>
#include <stdio.h>
#include <ipif.h>

void arpin (struct ep *pep) {
        int  iface;
        char merge_flag = FALSE;

        u_char  *ar_sha;
        IPaddr  *ar_spa;
        char    *ar_tha;
        IPaddr  *ar_tpa;
        IPaddr  tmp_ipaddr;
        struct arp_packet *packet;
        struct arp_entry  *pentry;
        struct arp_pinfo  *pinfo;
        
        packet = (struct arp_packet *) pep->ep_data;
        iface  = pep->ep_inif;
        
        if (packet->ar_op != hs2net(ARP_REQUEST) && 
            packet->ar_op != hs2net(ARP_REPLY)) {
                freebuf (pep); return;
        }

        if (packet->ar_htype != hs2net(nif[iface].ni_hwtype)) {
                freebuf (pep); return;
        }

        if (packet->ar_ptype != hs2net(ARP_PTYPE_IP)) {
                freebuf (pep); return;
        }

        if (packet->ar_hlen != nif[iface].ni_hwa.ha_len) {
                freebuf (pep); return;
        }

        if (packet->ar_plen != sizeof(IPaddr)) {
                freebuf (pep); return;
        }

        ar_sha = &(packet->ar_data);
        ar_spa = (IPaddr *)(ar_sha + packet->ar_hlen);
        ar_tha = ((char *) ar_spa) + packet->ar_plen;
        ar_tpa = (IPaddr *)(ar_tha + packet->ar_hlen);
        
        wait(Arp.arc_sem);

        for (pentry = Arp.arc_tbl; pentry; pentry = pentry->are_next)
                if (*ar_spa == pentry->are_ip && iface == pentry->are_iface)
                        break;
        if (pentry) {
                merge_flag = TRUE;
                blkcopy (pentry->are_hwaddr, ar_sha, packet->ar_hlen);
                while (pentry->are_pq) {
                        blkcopy (pentry->are_pq->ari_hwaddr, ar_sha, packet->ar_hlen);
                        if(mqsend (pentry->are_pq->ari_destq, (int) pentry->are_pq->ari_pep)==SYSERR) 
                                freebuf (pentry->are_pq->ari_pep);
                        pinfo = pentry->are_pq;
                        pentry->are_pq = pentry->are_pq->ari_next;
                        freebuf (pinfo);
                }
                pentry->are_pq = NULL;
                pentry->are_pq_tail = NULL;
                pentry->are_stat  = ARP_STAT_VALID;
                pentry->are_revalid = FALSE;
                pentry->are_timer = ARP_TIMER_REVALID;
        }

        if (*ar_tpa != ipif[iface].ipif_addr) {
                freebuf (pep); signal (Arp.arc_sem); return;
        }

        if (merge_flag == FALSE) {
                pentry = (struct arp_entry *) nbgetbuf(Arp.arc_entrypool);
                if ((int) pentry != SYSERR) {
                        pentry->are_stat = ARP_STAT_VALID;
                        pentry->are_revalid = FALSE;
                        pentry->are_ip = *ar_spa;
                        pentry->are_iface = iface;
                        bzero (pentry->are_hwaddr, ARP_MAXADDRLEN);
                        blkcopy (pentry->are_hwaddr, ar_sha, packet->ar_hlen);
                        pentry->are_pq = NULL;
                        pentry->are_pq_tail = NULL;
                        pentry->are_next = Arp.arc_tbl;
                        pentry->are_timer = ARP_TIMER_REVALID;
                        pentry->are_retry = -1;
                        Arp.arc_tbl = pentry;
                }
        }
        
        signal (Arp.arc_sem);
        
        if (packet->ar_op == hs2net(ARP_REQUEST)) {
                packet->ar_op = hs2net(ARP_REPLY);
                tmp_ipaddr = *ar_spa;
                *ar_spa = *ar_tpa;
                *ar_tpa = tmp_ipaddr;
                
                blkcopy (pep->ep_dst, ar_sha, packet->ar_hlen);  /* target hw address in eth frame */
                blkcopy (ar_tha, ar_sha, packet->ar_hlen);       /* target hw address */
                blkcopy (ar_sha, nif[iface].ni_hwa.ha_addr, packet->ar_hlen); /* source hw address */

		pep->ep_outif = iface;

                ni_out (pep);
        } else
                freebuf (pep);
        return;
}

