#include <conf.h>
#include <kernel.h>
#include <mq.h>
#include <network.h>
#include <arp.h>
#include <stdio.h>
#include <ipif.h>

int arplookup (IPaddr ipaddr, u_int iface,
               u_char* hwaddr, struct ep* pep, u_int destq) 
{
        struct arp_entry *pentry;
        struct arp_pinfo *pinfo;

        /* Check for broadcast */
	if (ipaddr == IPADDR_BCAST || ipaddr == ipif[iface].ipif_bcast) {
		blkcopy (hwaddr, nif[iface].ni_hwb.ha_addr,
			 nif[iface].ni_hwb.ha_len);
		return ARP_HIT;
	}

        if (IP_ISMULTI(ipaddr)) {
                if (iface == NI_PRIMARY) {
                        ipaddr = net2hl (ipaddr);
                        hwaddr[0] = 0x01; hwaddr[1] = 0x00; hwaddr[2] = 0x5E;
                        hwaddr[3] = (ipaddr >> 16) & 0x7;
                        hwaddr[4] = (ipaddr >> 8);
                        hwaddr[5] = ipaddr;
                } else {
                        blkcopy (hwaddr, nif[iface].ni_hwb.ha_addr,
                                nif[iface].ni_hwb.ha_len);
                }
                return ARP_HIT;
        }

        wait(Arp.arc_sem);

        for (pentry = Arp.arc_tbl; pentry; pentry = pentry->are_next)
                if (ipaddr == pentry->are_ip && iface == pentry->are_iface) 
                        break;

        if (pentry && (pentry->are_stat == ARP_STAT_VALID ||
                       pentry->are_stat == ARP_STAT_REVALID)) {
                blkcopy (hwaddr, pentry->are_hwaddr, ARP_MAXADDRLEN);
                if (pentry->are_timer < ARP_TIMER_REV_DELTA)
                        pentry->are_revalid = TRUE;
                signal(Arp.arc_sem);
                return (ARP_HIT);
        }

        /* check if the entry already exists (by now we know it's not valid */
        if (!pentry) {
                pentry = (struct arp_entry *) nbgetbuf(Arp.arc_entrypool);
                if ((int) pentry == SYSERR) {
                        freebuf (pep);
                        signal (Arp.arc_sem);
                        return (ARP_FAIL);
                }
                pentry->are_stat = ARP_STAT_REQUEST;
                pentry->are_ip = ipaddr;
                pentry->are_iface = iface;
                bzero (pentry->are_hwaddr, ARP_MAXADDRLEN);
                pentry->are_pq = NULL;
                pentry->are_pq_tail = NULL;
                pentry->are_next = Arp.arc_tbl;
                pentry->are_timer = ARP_TIMER_RETRY;
                pentry->are_retry = ARP_NRETRIES;
                Arp.arc_tbl = pentry;
                if(mqsend(Arp.arc_poutq, (int) pentry)!=SYSERR)
                        mqsend(Arp.arc_ctrlq, ARP_CMDPKTOUT);
        }
        
        /* get a new queue entry */
        pinfo = (struct arp_pinfo *) nbgetbuf(Arp.arc_pinfopool);
        if ((int) pinfo == SYSERR) {
                freebuf (pep);
                signal (Arp.arc_sem);
                return (ARP_FAIL);
        }

        /* enqueue the packet, and return MISS */
        if(pep) {
                pinfo->ari_hwaddr = hwaddr;
                pinfo->ari_pep = pep;
                pinfo->ari_destq = destq;
                pinfo->ari_next = NULL;
                if(pentry->are_pq==NULL) { 
                        pentry->are_pq = pinfo;
                        pentry->are_pq_tail = pinfo;
                } else {
                        pentry->are_pq_tail->ari_next = pinfo;
                        pentry->are_pq_tail = pinfo;
                }
        }
        signal(Arp.arc_sem);
        return ARP_MISS;
}
