#include <conf.h>
#include <kernel.h>
#include <arp.h>
#include <mq.h>
#include <timer.h>
#include <stdio.h>

void arpdel (struct arp_entry *pentry) {
        struct arp_entry *prev;
        struct arp_pinfo *pinfo;

        while (pentry->are_pq) {
                if(pentry->are_pq->ari_pep != NULL) 
                        freebuf (pentry->are_pq->ari_pep);
                pinfo = pentry->are_pq;
                pentry->are_pq = pentry->are_pq->ari_next;
                freebuf (pinfo);
        }

        if (Arp.arc_tbl == pentry)
                Arp.arc_tbl = pentry->are_next;
        else {
                for (prev = Arp.arc_tbl; prev->are_next != pentry;
                     prev = prev->are_next);
                prev->are_next = pentry->are_next;
        }

        freebuf (pentry);
}

void arptimer() {
        struct arp_entry *pentry;

        wait (Arp.arc_sem);
        for (pentry = Arp.arc_tbl; pentry; pentry = pentry->are_next) {
                if (--pentry->are_timer != 0) continue;
                switch (pentry->are_stat) {
                        case ARP_STAT_REQUEST:
                                if (--pentry->are_retry != 0) {
                                        pentry->are_timer = ARP_TIMER_RETRY;
                                        if(mqsend (Arp.arc_poutq, (int) pentry)!=SYSERR)
                                                mqsend (Arp.arc_ctrlq, ARP_CMDPKTOUT);
                                } else arpdel (pentry);
                                break;
                        case ARP_STAT_VALID:
                                if (pentry->are_revalid) {
                                        pentry->are_stat = ARP_STAT_REVALID;
                                        pentry->are_timer = ARP_TIMER_RETRY;
                                        if(mqsend (Arp.arc_poutq, (int) pentry)!=SYSERR)
                                                mqsend (Arp.arc_ctrlq, ARP_CMDPKTOUT);
                                } else 
                                        arpdel (pentry);
                                break;
                        case ARP_STAT_REVALID:
                                arpdel (pentry);
                                break;
                }
        }
        signal (Arp.arc_sem);
        tmset (ARP_TIMER_QUANTUM, Arp.arc_ctrlq, ARP_CMDTIMER);
}

