#include <conf.h>
#include <kernel.h>
#include <i386.h>
#include <network.h>
#include <proc.h>
#include <3c905.h>
#include <pci.h>
#include <othernet.h>
#include <stdio.h>

/*#define DEBUG*/
/*#define DEBUGOTHER*/
#define MAX_RCV			10		/* maximum packets per int */

SYSCALL _3c905_demux (dev) 
struct dev_3c905* dev;
{
    struct eth_pd* epd;
    struct ep*     pep;
    unsigned long  status;
    int len, nopackets, ifnum;

    nopackets = MAX_RCV;

    while (nopackets--) {
        epd = &eth_rxring[dev->ed_rxbegin];
        status = epd->pd_status;

        if (! (status & _3C905_FLG_UPCOMPLETE))
            break;

        if (status & _3C905_FLG_UPERROR) {
#ifdef DEBUG
            kprintf("3c905: packet error\n");
#endif
            epd->pd_status = 0;
            epd->pd_length = _3C905_FLG_LASTFRAG | EP_MAXLEN;
        } else {
            if (epd->pd_buffer) {
                dev->ed_state &= ~_3C905_STT_RECLAIM;
                len = status & _3C905_MSK_PKTLENGTH;
                pep = (struct ep*) (epd->pd_buffer - sizeof(struct ehx));
                pep->ep_inif  = dev->ed_ifnum;
                pep->ep_len  = len;
                pep->ep_type = net2hs(pep->ep_type);
#ifdef DEBUG
                {
                    int i;

                    kprintf("ETHER: dst %02x", pep->ep_dst[0]);
                    for (i=1; i<EP_ALEN; ++i)
                        kprintf(":%02x", pep->ep_dst[i]);
                    kprintf(" src %02x", pep->ep_src[0]);
                    for (i=1; i<EP_ALEN; ++i)
                        kprintf(":%02x", pep->ep_src[i]);
                    kprintf("\nETHER: type %x\n", pep->ep_type);
                }
#endif
                /*
                 * pass it up
                 */
                ifnum = dev->ed_ifnum;

#ifdef XPA
                /*
                 * if XPA flag is set, pass the frame to user-defined function
                 */
                if (dev->ed_xpa)
                    dev->ed_xpafunc(pep);
#endif

                /* check for a process waiting to read */
                if (dev->ed_rpid != BADPID) {
#ifdef DEBUG
                    kprintf("3c905: passed packet to proc %d\n", dev->ed_rpid);
#endif
                    /* pass the pep to the reading process */
                    if (enq(dev->ed_inq, pep, 0) < 0) {
                        nif[ifnum].ni_idiscard++;
#if defined(PRINTERR) || defined(DEBUG)
                        kprintf("3c905: couldn't enq()\n");
#endif
                        freebuf(pep);
                        return(SYSERR);
                    }

                    /* wake up the process */
                    ready(dev->ed_rpid, RESCHNO);
                    dev->ed_rpid = BADPID;
                    return(OK);
                }
                /*
                 * pass it to upper layer; may cause context switch
                 */
#if Noth > 0
                {
                    struct otblk *otptr;

                    /*
                     * The following code is for simulation Othernets
                     */
                    int i;

                    ifnum = -1;
                    /*
                     * Determine the incoming interface
                     */
                    for (i = 1; i < NIF; i++) {
                        if (nif[i].ni_state == NIS_DOWN)
                            continue;

                        /* Spring 99:
                           For smaller MAC address simulation we replace the
                           source and destionation addresses with the shorter
                           ones.
                        */
 
                        /* The check must be changed. We cannot check the NIF 
                           structures because these contain the perceived 
                           addresses. The Ether mcast addresses we need are 
                           stored in the otblk structures.
                        */
                        if (nif[i].ni_dev == ETHER) {
                          if (blkequ(nif[i].ni_hwa.ha_addr, pep->ep_dst, 
                                     EP_ALEN) ||
                              blkequ(nif[i].ni_hwb.ha_addr, pep->ep_dst, 
                                     EP_ALEN)) {
 
                            ifnum = i;
                            break;
                          }
                        }
 
                        /* else try othernets */
                        otptr = (struct otblk *) devtab[nif[i].ni_dev].dvioblk;
 
                        if (blkequ(otptr->ot_baddr, pep->ep_dst, EP_ALEN) ||
                            blkequ(otptr->ot_paddr, pep->ep_dst, EP_ALEN)) {
 
                          ifnum = i;
 
                          /* Note: For now no need to fix up source address.
                             We are not translating the source address; 
                             allowing the bogus source addresses to be used. 
                             This helps the groups in sniffing.
                          */
 
                          /* Figure out if it is the uniticast or broadcast 
                             address 
                          */
 
                          if (blkequ(otptr->ot_paddr, pep->ep_dst, EP_ALEN)) {
 
                            /* It was the "unicast" multicast address.
                               Fix up accordingly by replaceing it with the
                               perceived address.
                            */

                            bzero(pep->ep_dst, EP_ALEN);
                            blkcopy(pep->ep_dst, nif[i].ni_hwa.ha_addr, 
                                                 nif[i].ni_hwa.ha_len);
 
                          }
                          else {
                            /* It was the "broadcast" multicast address.
                               Fix up accordingly.
                            */
                            bzero(pep->ep_dst, EP_ALEN);
                            blkcopy(pep->ep_dst, nif[i].ni_hwb.ha_addr, 
                                                 nif[i].ni_hwb.ha_len);
                          } /* else */
 
#ifdef DEBUGOTHER
                          {
                              int i;

                              kprintf("OTHER: dst %02x", pep->ep_dst[0]);
                              for (i=1; i<EP_ALEN; ++i)
                                  kprintf(":%02x", pep->ep_dst[i]);
                              kprintf(" src %02x", pep->ep_src[0]);
                              for (i=1; i<EP_ALEN; ++i)
                                  kprintf(":%02x", pep->ep_src[i]);
                              kprintf("\n");
                          }
#endif
                          /* Done! */
                          break;
                        } /* if */
 
                    } /* for */
 
                    /* If we didn't match any of the interfaces bcast or
                       "unicast" addresses this may be an IP multicast packet.
                    */
 
                    if (ifnum >= NIF || ifnum < 0) {
                      /* This code is specific to the Spring 99 setup */

                      /* We assume othernet 1 has address len 6, othernet 2 
                         doesnt support MC 
                      */
 
                      /* Check the first 4 bytes for standard mapping 
                         If they match assume came from Xinunet 
                      */
                      if(0x01005E00 == (net2hl(*((unsigned int *)pep->ep_dst)) 
			 & 0xffffff80))
                        ifnum = ((struct ethdev_comm *)
			        devtab[ETHER].dvioblk)->edc_ifnum;
 
                      /* If bing ID is in second octet then assume othernet 2
                         Note that we cannot use BINGID 0 because of this
                      */
                      if((0x01005E00 | (bingid << 16)) == 
                         (net2hl(*((unsigned int *)pep->ep_dst)) & 0xffffff80))
                        ifnum = ((struct otblk *)
                                devtab[OTHER1].dvioblk)->ot_intf;
                    }
                }
#endif

                if (ifnum<0 || ifnum>=NIF || (nif[ifnum].ni_state == NIS_DOWN))
                    freebuf(pep);
                else {
                    /*
                     * may cause context switch
                     */
                    pep->ep_inif = ifnum;
                    ni_in(&nif[ifnum], pep, pep->ep_len);
                }
            }

            epd->pd_buffer = (void*) GETBUF(Net.netpool);
            if ((int)epd->pd_buffer == SYSERR || epd->pd_buffer == 0) {
#ifdef DEBUG
                kprintf("ethdemux: no buffer\n");
#endif
                epd->pd_buffer = NULL;
                dev->ed_state |= _3C905_STT_RECLAIM;
                break;

            }
            epd->pd_status = 0;
            epd->pd_buffer = (void*) epd->pd_buffer + sizeof(struct ehx);
            epd->pd_length = _3C905_FLG_LASTFRAG | EP_MAXLEN;
            dev->ed_state &= ~_3C905_STT_RECLAIM;
        }
        dev->ed_rxbegin = (dev->ed_rxbegin + 1) % _3C905_RXRING;
    }
    status = inl(dev->ed_iobase + _3C905_OFF_UPPKTSTATUS);
    if (status & _3C905_FLG_UPSTALLED)
        _3C905_CMD(dev, _3C905_CMD_UPUNSTALL, 0);
    return(OK);
}
