#include <conf.h>
#include <kernel.h>
#include <i386.h>
#include <proc.h>
#include <network.h>
#include <3c905.h>
#include <pci.h>
#include <q.h>
#include <stdio.h>

/*#define DEBUG*/

extern int ioerr();

extern int _3c905_int();
extern int _3c905_read();
extern int _3c905_write();
extern int _3c905_cntl();
extern int _3c905_rawrite();

struct dev_3c905 dev_eth;
struct eth_pd*   eth_txring;
struct eth_pd*   eth_rxring;
struct eth_pd    txring[_3C905_TXRING + 1];
struct eth_pd    rxring[_3C905_RXRING + 1];

SYSCALL _3c905_init (pdev) 
struct devsw* pdev;
{
    struct dev_3c905* dev;
    struct ethdev*    ped=NULL;
    unsigned short    status;
    unsigned char*    buf;
    int i;

    pdev->dvinit  = _3c905_init;
    pdev->dvopen  = ioerr;
    pdev->dvclose = ioerr;
    pdev->dvread  = _3c905_read;
    pdev->dvwrite = _3c905_write;
    pdev->dvseek  = ioerr;
    pdev->dvgetc  = ioerr;
    pdev->dvputc  = ioerr;
    pdev->dvcntl  = _3c905_cntl;
    pdev->dviint  = _3c905_int;
    pdev->dvoint  = _3c905_int;
    pdev->dvioblk = (char*) &dev_eth.eddev_comm;

    dev = &dev_eth;
    dev->ed_pdev  = pdev;
    dev->ed_outq  = 0;					/* don't need that */
    dev->ed_inq   = newq(ETINQSZ,  QF_NOWAIT);
    dev->ed_rpid  = BADPID;
    dev->ed_rsem  = screate(1);
    dev->ed_ifnum = 0;
#ifdef XPA
    dev->ed_xpa     = 0;
    dev->ed_xpafunc = 0;
#endif
    dev->ed_rawrite = _3c905_rawrite;
    dev->ed_descr = "3COM 3c905C";
    sprintf(dev->ed_aka, "xl%d", pdev->dvminor);
    dev->ed_state = 0;
    ped->ed_mcc = 0;
    ped->ed_mcset = 0;          /* no multicast */
    ped->ed_proms = 0;

    /* read config information */
    pci_bios_read_config_dword(dev->ed_pcidev,
                               _3C905_PCI_IOBASE,
                               (unsigned long *)&dev->ed_iobase);
    pci_bios_read_config_dword(dev->ed_pcidev,
                               _3C905_PCI_MEMBASE,
                               (unsigned long *)&dev->ed_membase);
    pci_bios_read_config_byte (dev->ed_pcidev,
                               _3C905_PCI_IRQ,
                               &dev->ed_irq);

    /* the low bit is set to indicate I/O */
    dev->ed_iobase &= ~1;

    /* if mem address is below 1 MB */
    dev->ed_membase &= ~2;

    /* enable PCI bus master */
    pci_bios_read_config_word(dev->ed_pcidev, PCI_COMMAND, (unsigned short *)&status);
    status |= PCI_BUSMASTER;
    pci_bios_write_config_word(dev->ed_pcidev, PCI_COMMAND, status);

    /* read the MAC address */
    _3C905_SEL(dev, 2);
    *((unsigned short*) &dev->ed_paddr[0]) = inw(dev->ed_iobase + 0x00);
    *((unsigned short*) &dev->ed_paddr[2]) = inw(dev->ed_iobase + 0x02);
    *((unsigned short*) &dev->ed_paddr[4]) = inw(dev->ed_iobase + 0x04);

/*
#ifdef BOOT_MONITOR
    *((unsigned short*) &dev->ed_hwa[0]) |= 0xFF01;
#endif
*/

    /* assume a1s broadcast is set on card */
    for (i = 0; i < EP_ALEN; i++)
    	dev->ed_bcast[i] = ~0;

    /* initialize TX/RX rings */
    eth_txring = (struct eth_pd*) (((unsigned long) txring + 0xF) & ~0xF);
    eth_rxring = (struct eth_pd*) (((unsigned long) rxring + 0xF) & ~0xF);
    for (i = 0; i < _3C905_RXRING; i++) {
        eth_rxring[i].pd_next   = &eth_rxring[(i + 1) % _3C905_RXRING];
        eth_rxring[i].pd_status = 0;
        buf = (unsigned char*) GETBUF(Net.netpool); 
        if ((int)buf == SYSERR || buf == 0) {
            kprintf("3c905: No buffer in ethinit()\n");
            panic("...");
        }
        eth_rxring[i].pd_buffer = (void*) buf + sizeof(struct ehx);
        eth_rxring[i].pd_length = _3C905_FLG_LASTFRAG | EP_MAXLEN;
    }
    dev->ed_rxbegin = 0;
    bzero(eth_txring, sizeof(struct eth_pd) * _3C905_TXRING);
    dev->ed_txbegin = 0;
    dev->ed_txend   = _3C905_TXRING - 1;

    /* reset nic */
    _3c905_cmd(dev, _3C905_CMD_RXRESET, 0x7);
    _3c905_cmd(dev, _3C905_CMD_TXRESET, 0x3);

    /* enable interrupts */
    set_evec(dev->ed_irq + IRQBASE, (u_long)_3c905_int);
    _3C905_CMD(dev, _3C905_CMD_SETINDICATION, _3C905_MSK_INTERRUPTS);
    _3C905_CMD(dev, _3C905_CMD_SETINTERRUPT, _3C905_MSK_INTERRUPTS);

    /* enable receiving and transmission */
    _3C905_CMD(dev, _3C905_CMD_SETRXFILTER, _3C905_MSK_RXFILTER);
    _3c905_cmd(dev, _3C905_CMD_UPSTALL, 0);
    outl(dev->ed_iobase + _3C905_OFF_UPLISTPTR, (int)eth_rxring);
    _3C905_CMD(dev, _3C905_CMD_UPUNSTALL, 0);
    _3C905_CMD(dev, _3C905_CMD_RXENABLE, 0);
    _3C905_CMD(dev, _3C905_CMD_TXENABLE, 0);

    /* set the OS structures */
    pdev->dvcsr  = dev->ed_iobase;
    pdev->dvivec = dev->ed_irq + IRQBASE;
    pdev->dvovec = dev->ed_irq + IRQBASE;
    return(OK);
}
