#include <conf.h>
#include <kernel.h>
#include <i386.h>
#include <network.h>
#include <3c905.h>
#include <pci.h>
#include <stdio.h>

/*#define DEBUG*/

SYSCALL _3c905_intr() {
    STATWORD          ps;
    struct dev_3c905* dev;
    struct eth_pd*    epd;
    unsigned long     intst, txst;

    disable(ps);

    dev = &dev_eth;

    _3C905_CMD(dev, _3C905_CMD_SETINTERRUPT, 0);

    /* get status */
    intst = inw(dev->ed_iobase + _3C905_OFF_INTSTATUS);

    /* not our interrupt */
    if (! (intst & _3C905_FLG_INTERRUPT)) {
        _3C905_CMD(dev, _3C905_CMD_SETINTERRUPT, _3C905_MSK_INTERRUPTS);
        restore(ps);
        return(OK);
    }

    /* acknowledge interrupt */
    _3C905_CMD(dev, _3C905_CMD_ACKINTERRUPT, _3C905_FLG_INTERRUPT);

    while(1) {
        /* get status */
        intst = inw(dev->ed_iobase + _3C905_OFF_INTSTATUS) & 
		_3C905_MSK_INTERRUPTS;

#ifdef DEBUG
        kprintf("3c905: is=0x%x upp=0x%x dnp=0x%x\n",
                intst, 
                inl(dev->ed_iobase + _3C905_OFF_UPLISTPTR),
                inl(dev->ed_iobase + _3C905_OFF_DNLISTPTR));
#endif

        if (intst == 0)
            break;

        if (intst & _3C905_FLG_UPCOMPLINT) {
            _3C905_CMD(dev, _3C905_CMD_ACKINTERRUPT, _3C905_FLG_UPCOMPLINT);
            _3c905_demux(dev);
        }
        if (intst & _3C905_FLG_TXCOMPLINT) {
            _3c905_xintr(dev);
            txst  = inb(dev->ed_iobase + _3C905_OFF_TXSTATUS);
            outb(dev->ed_iobase + _3C905_OFF_TXSTATUS, 1);
        }
        if (dev->ed_state & _3C905_STT_RECLAIM) {
            while (1) {
                epd = &eth_rxring[dev->ed_rxbegin];
                if (epd->pd_status == 0 || epd->pd_buffer != 0) {
                    dev->ed_state &= ~_3C905_STT_RECLAIM;
                    break;
                }
		epd->pd_buffer = (void*) GETBUF(Net.netpool);
                if (epd->pd_buffer == 0 || (int)epd->pd_buffer == SYSERR){
                    epd->pd_buffer = 0;
                    break;
                }
                epd->pd_status = 0;
                epd->pd_buffer = (void*) epd->pd_buffer + sizeof(struct ehx);
                epd->pd_length = _3C905_FLG_LASTFRAG | EP_MAXLEN;
                dev->ed_rxbegin = (dev->ed_rxbegin + 1) % _3C905_RXRING;
            }
        }
    }
#ifdef DEBUG
    kprintf("3c905: intr leave\n");
#endif
    _3C905_CMD(dev, _3C905_CMD_SETINTERRUPT, _3C905_MSK_INTERRUPTS);
    restore(ps);
    return(OK);
}

SYSCALL _3c905_xintr(dev)
struct dev_3c905* dev;
{
    struct eth_pd* epd;
    struct ep*     pep;
    unsigned long  status;

    while(1) {
        epd = &eth_txring[dev->ed_txbegin];
        status = epd->pd_status;

        if (! (status & _3C905_FLG_DNCOMPLETE))
            break;

        status = inb(dev->ed_iobase + _3C905_OFF_TXSTATUS);
        /* write to advance */
        outb(dev->ed_iobase + _3C905_OFF_TXSTATUS, status);

        if (status & _3C905_MSK_TXERROR) {
#ifdef DEBUG
            kprintf("3c905: tx error\n");
#endif
            if (status & (_3C905_FLG_ERRTXSTATUSOVF|_3C905_FLG_ERRTXMAXCOLL)) {
                _3C905_CMD(dev, _3C905_CMD_TXENABLE, 0);
            }
        }

        epd->pd_status = 0;
        pep = (struct ep*) (epd->pd_buffer - sizeof(struct ehx));
        freebuf(pep);
        dev->ed_txbegin = (dev->ed_txbegin + 1) % _3C905_TXRING;
    }
    return(OK);
}
