#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <i386.h>
#include <3c905.h>
#include <stdio.h>

/*#define DEBUG*/

LOCAL int _3c905_mcast(), hashbit();

/*------------------------------------------------------------------------
 * _3c905_mcadd - add multicast addresses
 *------------------------------------------------------------------------
 */
int _3c905_mcadd(dev, mca)
struct dev_3c905	*dev;
Eaddr			mca;
{
    int		i;

    for (i = 0; i < dev->ed_mcc; ++i) {
      /* Argh, the length field was left off!
	 1-29-99 MHE
      */
        if (blkequ(mca, dev->ed_mca[i], EP_ALEN)) {
            return OK;  /* already there */
        }
    }
    blkcopy(dev->ed_mca[i], mca, EP_ALEN);
    dev->ed_mcc++;
    dev->ed_mcset = 1;

    if (_3c905_mcast(dev, dev->ed_mca, dev->ed_mcc) == SYSERR)
	return SYSERR;
    
#ifdef DEBUG
    kprintf("mcadd:\n");
    mcdump(dev);
#endif

    return OK;
}

/*------------------------------------------------------------------------
 * _3c905_mcdel - delete multicast addresses
 *------------------------------------------------------------------------
 */
int _3c905_mcdel(dev, mca)
struct dev_3c905	*dev;
Eaddr			mca;
{
    int		i;

    for (i = 0; i < dev->ed_mcc; ++i) {
        if (blkequ(mca, dev->ed_mca[i], EP_ALEN))
            break;
    }

    if (i == dev->ed_mcc) {
        return OK;	/* not found */
    }
    
    i++;
    for (; i < dev->ed_mcc; ++i)
        blkcopy(dev->ed_mca[i-1], dev->ed_mca[i], EP_ALEN);

    dev->ed_mcc--;

    if (_3c905_mcast(dev, dev->ed_mca, dev->ed_mcc) == SYSERR)
	return SYSERR;

#ifdef DEBUG
    kprintf("mcdel:\n");
    mcdump(dev);
#endif
    
    return OK;
}

/*-------------------------------------------------------------------------
 * _3c905_mcast - setup multicast for 3C905C
 *-------------------------------------------------------------------------
 */
LOCAL int _3c905_mcast(dev, mc_addr, mc_count)
struct dev_3c905	*dev;
Eaddr			*mc_addr;
u_short			mc_count;
{
    int		i;	 

    /* stop receiver */
    _3c905_cmd(dev, _3C905_CMD_RXDISABLE, 0);

    /* clear all addresses */
    for (i = 0; i < _3C905_MSK_MCASTFILTER + 1; i++)
        _3C905_CMD(dev, _3C905_CMD_SETMCASTBIT, i);

    /* set all addresses */
    for (i = 0; i < mc_count; i++)
        _3C905_CMD(dev, _3C905_CMD_SETMCASTBIT, _3C905_FLG_MCASTENABLE | 
						hashbit(mc_addr[i]));

    /* enable receive */
    _3C905_CMD(dev, _3C905_CMD_RXENABLE, 0);

    return OK;
}

/*-------------------------------------------------------------------------
 * hashbit - calculate which bit to set - (from 3com manual)
 *-------------------------------------------------------------------------
 */
LOCAL int hashbit(mca)
Eaddr mca;
{
    unsigned long crc, carry;
    unsigned int i, j;
    unsigned char byte;

    crc = 0xffffffff;

    for (i = 0; i < 6; i++) {
        byte = mca[i];
        for (j = 0; j < 8; j++) {
            carry = ((crc & 0x80000000) ? 1 : 0) ^ (byte & 0x01);
            crc <<= 1;
            byte >>= 1;
            if (carry)
                crc = (crc ^ 0x04c11db6) | carry;
        }
    } 
    return crc & _3C905_MSK_MCASTFILTER;
}
