#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <i386.h>
#include <eepro.h>
#include <stdio.h>

/*#define DEBUG*/

/*------------------------------------------------------------------------
 * eep_cntl - handle EtherExpress control() functions
 *------------------------------------------------------------------------
 */
int eep_cntl(pdev, func, arg)
struct devsw    *pdev;
int             func;
u_char            *arg;
{
    STATWORD    ps;
    struct      ethdev   *ped = (struct ethdev *)pdev->dvioblk;
    u_short     iobase;
    int         rv;

    iobase = pdev->dvcsr;

    disable(ps);
    
    switch (func) {
    case EPC_PROMON:    /* turn on promiscuous mode */
        ped->ed_proms = 1;

	if ((rv = set_proms_mode(ped, iobase, ped->ed_proms)) == OK)
#ifdef DEBUG
	    kprintf("%s: promiscuous mode enabled\n", ped->ed_aka);
#endif
        break;

    case EPC_PROMOFF:   /* turn off promiscuous mode */
        ped->ed_proms = 0;
	if ((rv = set_proms_mode(ped, iobase, ped->ed_proms)) == OK)
#ifdef DEBUG
	    kprintf("%s: promiscuous mode disabled\n", ped->ed_aka);
#endif
        break;

    case EPC_MADD:	/* add a multicast address */
	rv = eep_mcadd(ped, arg, iobase);
        break;

    case EPC_MDEL:
	rv = eep_mcdel(ped, arg, iobase);
        break;

    default:
        rv = SYSERR;
    }

    restore(ps);

    if (rv == SYSERR)
        kprintf("eep_cntl: ** func=%d failed **\n", func);
    return rv;   
}


/*-------------------------------------------------------------------------
 * set_proms_mode - 
 *-------------------------------------------------------------------------
 */
int set_proms_mode(ped, iobase, on)
struct ethdev *ped;
u_short iobase;
u_short on;		/* 1: turn on promiscuous mode */
{
    u_char mode;
    int rv;
    
    outb(iobase, EEP_BANK2_SELECT); /* be CAREFULL, BANK 2 now */
    mode = inb(iobase + EEP_REG2);
#ifdef DEBUG
    kprintf("eep_cntl: promiscuous mode, mode=0x%x\n", mode);
#endif
    if (on)
	mode |= EEP_PRMSC_Mode;
    else
	mode &= ~EEP_PRMSC_Mode;
	
    outb(iobase + EEP_REG2, mode);
    
    /* writing config register (reg. 3) to complete the update */
    mode = inb(iobase + EEP_REG3);
    outb(iobase + EEP_REG3, mode);
    outb(iobase, EEP_BANK0_SELECT); /* Return to BANK 0 */

    /* wait for execution unit to become idle and issue a select reset */
    if (wait_for_exec(iobase)) {
	outb(iobase, EEP_SEL_RESET_CMD);
	DELAY(100);

	/*
	 * reset DMA register (they were wiped by the Selective reset)
	 * set RCV CAR/BAR, TX CAR/BAR, and the receive stop regiser
	 */
	/* Initialize RCV CAR/BAR, stop register */
	outw(iobase + EEP_RCV_BAR, ped->ed_txbuf_size);
	outw(iobase + EEP_RCV_STOP, ped->ed_mem_size - 1);
	ped->ed_rx_start = ped->ed_txbuf_size;
    
	outw(iobase + EEP_XMT_BAR, 0);
	ped->ed_tx_start = ped->ed_tx_end = 0;
	rv = OK;
    }
    else
	rv = SYSERR;

    /* enable receive */
    outb(iobase, EEP_RCV_ENABLE_CMD);
    return (rv);
}



