#include <conf.h>
#include <kernel.h>
#include <proc.h>
#include <network.h>
#include <i386.h>
#include <eepro.h>
#include <othernet.h>
#include <stdio.h>

#define MAX_RCV		5	/* max. # of frames received in one shot */

LOCAL int rcv_frame(struct ethdev *ped, u_short port, u_short len);

/*#define DEBUG*/
/*#define DEBUGOTHER*/
#define PRINTERR
/*------------------------------------------------------------------------
 * eep_demux - receive frames from an Intel EtherExpress
 *------------------------------------------------------------------------
 */
int eep_demux(ped, iobase)
struct ethdev	*ped;
u_short		iobase;
{
    u_short port = iobase + EEP_IO_PORT;
    u_short rcv_event, rcv_status, rcv_next, rcv_size, rcv_stop;
    int max_rcv = MAX_RCV;

#ifdef DEBUG
    kprintf("eep_demux: >>> start=%d\n", ped->ed_rx_start);
#endif

    while (1) {
	outw(iobase + EEP_HOST_ADDRESS_REG, ped->ed_rx_start);
	rcv_event = inw(port);
        rcv_status = inw(port);

	if (!((rcv_status & EEP_RX_OK) && (rcv_event & EEP_RCV_DONE)))
	    break;
	
        rcv_next = inw(port);
        rcv_size = inw(port);
	
#ifdef DEBUG
	kprintf("eep_demux: status=%x, next=%d, size=%d\n", rcv_status,
		rcv_next, rcv_size);
#endif
	    
	ped->ed_rx_start = rcv_next;

	if (rcv_size <= EP_MAXLEN) {
	    if (ped->ed_ifnum > 0 && ped->ed_ifnum < NIF)
		nif[ped->ed_ifnum].ni_nin++;
	
	    /*
	     * Note: may cause context switch
	     */
	    rcv_frame(ped, port, rcv_size);

	}
#ifdef PRINTERR
	else
	    kprintf("eep_demux: frame size too big! (len=%d)\n", rcv_size);
#endif
	
	/*
	 * set RX STOP address; make sure it won't fall in the TX buffer
	 */
	rcv_stop = ped->ed_rx_start - 2;
	if (rcv_stop < ped->ed_txbuf_size)
	    rcv_stop += ped->ed_rxbuf_size;
	
	outw(iobase + EEP_RCV_STOP, rcv_stop);
	
	if (--max_rcv == 0)
	    break;
    } /* while */

#ifdef DEBUG
    kprintf("eep_demux: exit start=%d\n", ped->ed_rx_start);
#endif
    return(OK);
}

/*-------------------------------------------------------------------------
 * rcv_frame - 
 *-------------------------------------------------------------------------
 */
LOCAL int rcv_frame(struct ethdev *ped, u_short port, u_short len)
{
    struct ep   *pep;
    int		ifnum;
    
    pep = (struct ep *)nbgetbuf(Net.netpool);
    if (pep == 0 || pep == (struct ep *) SYSERR) {
        if (ped->ed_ifnum > 0 && ped->ed_ifnum < NIF)
            nif[ped->ed_ifnum].ni_inobuf++;
#ifdef PRINTERR
        kprintf("rcv_frame: ?? no buffer\n");
#endif
        return(OK);
    }

#ifdef DEBUG
    kprintf("rcv_frame: len=%d\n", len);
#endif

    pep->ep_len = len;

    /*
     * copy packet to buffer
     */
    insw(port, (int)&pep->ep_eh, (len + 1) >> 1);
    pep->ep_type = net2hs(pep->ep_type);

#ifdef DEBUG
    {
        int i;

        kprintf("ETHER: dst %02x", pep->ep_dst[0]);
        for (i=1; i<EP_ALEN; ++i)
            kprintf(":%02x", pep->ep_dst[i]);
        kprintf(" src %02x", pep->ep_src[0]);
        for (i=1; i<EP_ALEN; ++i)
            kprintf(":%02x", pep->ep_src[i]);
        kprintf("\nETHER: type %x\n", pep->ep_type);
    }
#endif

    /*
     * pass it up
     */
    ifnum = ped->ed_ifnum;

#ifdef XPA
    /*
     * if XPA flag is set, pass the frame to user-defined function
     */
    if (ped->ed_xpa) {
        ped->ed_xpafunc(pep);
    }
#endif

    /* check for a process waiting to read */
    if (ped->ed_rpid != BADPID) {
#ifdef DEBUG
	kprintf("rcv_frame: passed packet to proc %d\n", ped->ed_rpid);
#endif
	/* pass the pep to the reading process */
	if (enq(ped->ed_inq, pep, 0) < 0) {
	    nif[ifnum].ni_idiscard++;
#if defined(PRINTERR) || defined(DEBUG)
	    kprintf("rcv_frame: couldn't enq()\n");
#endif
	    freebuf(pep);
	    return(SYSERR);
	}

	/* wake up the process */
	ready(ped->ed_rpid, RESCHNO);
	ped->ed_rpid = BADPID;
	return(OK);
    }

    /*
     * pass it to upper layer; may cause context switch
     */
#if     Noth > 0
    {
      struct otblk	*otptr;

	/*
	 * The following code is for simulation Othernets
	 */
	int i;

	ifnum = -1;
	/*
	 * Determine the incoming interface
	 */
	for (i = 1; i < NIF; i++) {

	    if (nif[i].ni_state == NIS_DOWN)
		continue;

	    /* Spring 99:
	       For smaller MAC address simulation we replace the 
	       source and destionation addresses with the shorter
	       ones.
	    */
		
	    /* The check must be changed. We cannot check the NIF structures
	       because these contain the perceived addresses. The Ether 
	       mcast addresses we need are stored in the otblk structures.
	    */
	    if (nif[i].ni_dev == ETHER) {
	      if (blkequ(nif[i].ni_hwa.ha_addr, pep->ep_dst, EP_ALEN) ||
		  blkequ(nif[i].ni_hwb.ha_addr, pep->ep_dst, EP_ALEN)) {
		
		ifnum = i;
		break;
	      }
	    }
	    
	    /* else try othernets */
	    otptr = (struct otblk *) devtab[nif[i].ni_dev].dvioblk;
	    
	    if (blkequ(otptr->ot_baddr, pep->ep_dst, EP_ALEN) ||
		blkequ(otptr->ot_paddr, pep->ep_dst, EP_ALEN)) {
	      
	      ifnum = i;
	      
	      /* Note: For now no need to fix up source address. 
		 We are not translating the source address; allowing
		 the bogus source addresses to be used. This helps
		 the groups in sniffing.
	      */

	      /* Figure out if it is the uniticast or broadcast address */

	      if (blkequ(otptr->ot_paddr, pep->ep_dst, EP_ALEN)) {
		
		/* It was the "unicast" multicast address. 
		   Fix up accordingly by replaceing it with the 
		   perceived address.
		*/

		bzero(pep->ep_dst, EP_ALEN);
		blkcopy(pep->ep_dst, nif[i].ni_hwa.ha_addr, nif[i].ni_hwa.ha_len);

	      }
	      else {
		/* It was the "broadcast" multicast address.
		   Fix up accordingly.
		*/		
		bzero(pep->ep_dst, EP_ALEN);
		blkcopy(pep->ep_dst, nif[i].ni_hwb.ha_addr, nif[i].ni_hwb.ha_len);
	      } /* else */

#ifdef DEBUGOTHER
    {
        int i;

        kprintf("ETHER(other): dst %02x", pep->ep_dst[0]);
        for (i=1; i<EP_ALEN; ++i)
            kprintf(":%02x", pep->ep_dst[i]);
        kprintf(" src %02x", pep->ep_src[0]);
        for (i=1; i<EP_ALEN; ++i)
            kprintf(":%02x", pep->ep_src[i]);
	kprintf("\n");
    }

#endif
	      /* Done! */
	      break;	      
	    } /* if */

	} /* for */

	/* If we didn't match any of the interfaces bcast or
	   "unicast" addresses this may be an IP multicast 
	   packet.
	*/

	if (ifnum >= NIF || ifnum < 0) {
	  /* This code is specific to the Spring 99 setup				*/
	  /* We assume othernet 1 has address len 6, othernet 2 doesnt support MC	*/
	  
	  /* Check the first 4 bytes for standard mapping	*/
	  /* If they match assume came from Xinunet		*/
	  if(0x01005E00 == (net2hl(*((unsigned int *)pep->ep_dst)) & 0xffffff80))
	    ifnum = ((struct ethdev_comm *)devtab[ETHER].dvioblk)->edc_ifnum;

	  /* If bing ID is in second octet then assume othernet 2	*/
	  /* Note that we cannot use BINGID 0 because of this		*/
	  if((0x01005E00 | (bingid << 16)) == (net2hl(*((unsigned int *)pep->ep_dst)) & 0xffffff80)) 
	    ifnum = ((struct otblk *)devtab[OTHER1].dvioblk)->ot_intf;
	}
    }
#endif
    
    if (ifnum < 0 || ifnum >= NIF || (nif[ifnum].ni_state == NIS_DOWN))
	freebuf(pep);
    else {
	/*
	 * may cause context switch
	 */
	pep->ep_inif = ifnum;
	ni_in(&nif[ifnum], pep, pep->ep_len);
    }
    return(OK);
}
