#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <eepro.h>
#include <stdio.h>

int eep_xintr(struct ethdev *, u_short);

/*#define DEBUG*/
/*#define DEBUG_TX*/
#define PRINTERR
/*------------------------------------------------------------------------
 * eep_intr - handle an EtherExpress PRO/10 device interrupt
 *------------------------------------------------------------------------
 */
int eep_intr()
{
    STATWORD    ps;
    struct 	ethdev	*ped;
    u_short	status;
    u_short	iobase;

    disable(ps);

    ped = &eep[0];
    iobase = ped->ed_pdev->dvcsr;

    while (1) {
	status = inb(iobase + EEP_STATUS_REG);

#ifdef DEBUG
	kprintf("eep_intr: status=0x%x\n", status);
#endif
	
	status &= 0x0f;		/* get the lower byte */
	
	if (status == 0) {
#ifdef DEBUG
	    kprintf("eep_intr: exit >>>>\n");
#endif
	    restore(ps);	/* done */
	    return(OK);
	}

	if (status & EEP_EXEC_INT) {
	    outb(iobase + EEP_STATUS_REG, EEP_EXEC_INT); /* ACK */
	}

	if (status & EEP_TX_INT) {
#ifdef DEBUG
	    kprintf("eep: TX int\n");
#endif

	    outb(iobase + EEP_STATUS_REG, EEP_TX_INT);	/* ACK */
	    
	    /* Process the status of transmitted packets */
	    eep_xintr(ped, iobase);
	}

	if (status & (EEP_RX_INT | EEP_RX_STP_INT)) {
#ifdef DEBUG
	    kprintf("eep: RX int\n");
#endif

	    outb(iobase + EEP_STATUS_REG, (EEP_RX_INT | EEP_RX_STP_INT));

	    /* Get the received packets */
	    eep_demux(ped, iobase);
	}
	
        /*
	 * if packets waiting in the output queue, then transmit it
	 */
	if (lenq(ped->ed_outq))
	    eep_wstrt(ped);
	
    } /* while */
}

/*-------------------------------------------------------------------------
 * eep_xintr - handles a TX interrupt
 *-------------------------------------------------------------------------
 */
int eep_xintr(struct ethdev *ped, u_short iobase)
{
    char restart = FALSE;
    u_short event, status, addr_port, io_port;
    struct netif *pni;

    addr_port = iobase + EEP_HOST_ADDRESS_REG;
    io_port = iobase + EEP_IO_PORT;
    
    pni = &nif[ped->ed_ifnum];
    pni->ni_tx_int++;

    while (ped->ed_tx_start != ped->ed_tx_end) {
	outw(addr_port, ped->ed_tx_start);
        event = inw(io_port);

#ifdef DEBUG_TX
	kprintf("eep_xintr: st=%d, end=%d, TX event=0x%x\n", 
	ped->ed_tx_start, ped->ed_tx_end, event);
#endif
        if ((event & I82595_TX_DONE_BIT) == 0)
	    break;
	
	status = inw(io_port);
	
#ifdef DEBUG_TX
	kprintf("eep_xintr: status=%x, start=%x\n", status, ped->ed_tx_start);
#endif

        if (status & EEP_TX_OK) {
	    pni->ni_tx_ok++;
	    if (status & EEP_TX_DEFFERED)
		pni->ni_tx_deffered++;
	    else if ((status & EEP_TX_COLLISION_MASK) == 1)
		pni->ni_tx_1_retry++;
	    else if ((status & EEP_TX_COLLISION_MASK) > 1)
		pni->ni_tx_n_retry++;
	}
        else {
#ifdef PRINTERR
	    kprintf("eep: TX error, status=0x%x\n", status);
#endif
	    restart = TRUE;
	    /*
	     * If we get here, we had a fatal transmit; we need to restart
	     * transmitter.
	     */
	    pni->ni_oerrors++;
	    if (status & EEP_TX_MAX_COLLISIONS)
		pni->ni_tx_max_collision++;
	    else if (status & EEP_TX_NO_CARRIER)
		pni->ni_tx_no_carrier++;
	    else if (status & EEP_TX_DMA_UNDERRUN)
		pni->ni_tx_dma_underrun++;
	}

	ped->ed_tx_start = inw(io_port);	/* next TX block */
	
	/*
	 * check to see if we need to restart transmitter
	 */
	if (restart) {
	    outw(iobase + EEP_XMT_BAR, ped->ed_tx_start);
	    if (ped->ed_tx_start != ped->ed_tx_end)
		outb(iobase, EEP_XMT_CMD);
	    restart = FALSE;
	}
    } /* while */
    return(OK);
}
