#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <i386.h>
#include <eepro.h>
#include <stdio.h>

/*#define DEBUG*/

LOCAL int setup_mcast();

/*------------------------------------------------------------------------
 * eep_mcadd - add multicast addresses for Intel EtherExpress16
 *------------------------------------------------------------------------
 */
int eep_mcadd(ped, mca, iobase)
struct ethdev	*ped;
Eaddr		mca;
u_short		iobase;
{
    int		i;

    for (i = 0; i < ped->ed_mcc; ++i) {
        if (blkequ(mca, ped->ed_mca[i], EP_ALEN)) {
            return OK;  /* already there */
        }
    }
    blkcopy(ped->ed_mca[i], mca, EP_ALEN);
    ped->ed_mcc++;
    ped->ed_mcset = 1;

    if (setup_mcast(ped, iobase, ped->ed_mca, ped->ed_mcc) == SYSERR)
	return SYSERR;
    
#ifdef DEBUG
    kprintf("ee_mcadd:\n");
    mcdump(ped);
#endif

    return OK;
}

/*------------------------------------------------------------------------
 * eep_mcdel - delete multicast addresses for Intel EtherExpress16
 *------------------------------------------------------------------------
 */
int eep_mcdel(ped, mca, iobase)
struct ethdev	*ped;
Eaddr		mca;
u_short		iobase;
{
    int		i;

    for (i = 0; i < ped->ed_mcc; ++i) {
        if (blkequ(mca, ped->ed_mca[i], EP_ALEN))
            break;
    }

    if (i == ped->ed_mcc) {
        return OK;	/* not found */
    }
    
    i++;
    for (; i < ped->ed_mcc; ++i)
        blkcopy(ped->ed_mca[i-1], ped->ed_mca[i], EP_ALEN);

    ped->ed_mcc--;

    if (setup_mcast(ped, iobase, ped->ed_mca, ped->ed_mcc) == SYSERR)
	return SYSERR;

#ifdef DEBUG
    kprintf("ee_mcadd:\n");
    mcdump(ped);
#endif
    
    return OK;
}

/*-------------------------------------------------------------------------
 * setup_mcast - 
 *-------------------------------------------------------------------------
 */
LOCAL int setup_mcast(ped, iobase, mc_addr, mc_count)
struct ethdev	*ped;
u_short		iobase;
Eaddr		*mc_addr;
u_short		mc_count;
{
    u_char	mask;
    u_short	*w;
    int		i, j, result = SYSERR;	 

    /* NOTE: assume in BANK 0 */
    
    /* stop receiver */
    outb(iobase+EEP_COMMAND_REG, EEP_STOP_RCV_CMD);

    /* save the current int. mask */
    mask = inb(iobase+EEP_INT_MASK_REG);

    /* disable all board interrupts */
    outb(iobase+EEP_INT_MASK_REG, EEP_ALL_MASK);

    /* wait for execution unit to be idle */
    if (wait_for_exec(iobase)) {
	/* EXEC is idle; set tx pointer to the beginning of the tx buffer */
	outw(iobase+EEP_XMT_BAR, 0);

	/* write mcast addresses to board memory and issue MC command */
	outw(iobase+EEP_HOST_ADDRESS_REG, 0);
	outw(iobase+EEP_IO_PORT, EEP_MC_SETUP);
	outw(iobase+EEP_IO_PORT, 0);
	outw(iobase+EEP_IO_PORT, 0);
	outw(iobase+EEP_IO_PORT, mc_count*EP_ALEN);
	
	for (j = 0; j < mc_count; j++) {
	    w = (u_short *)mc_addr[j];
	    for (i = 0; i < EP_ALEN/2; i++) {
		outw(iobase+EEP_IO_PORT, *w++);
	    }
	}

	if (eep_cmd(iobase, EEP_MC_SETUP, EEP_MC_SETUP) == OK) {
	    result = OK;
	    /*
	     * reset TX related variables
	     */
	    ped->ed_tx_start = ped->ed_tx_end = 0;
	}
    }
    else
	kprintf("eep_mcast: ** execution unit failed to stop **\n");

    /* enable receive */
    outb(iobase+EEP_COMMAND_REG, EEP_RCV_ENABLE_CMD);

    /* restore board interrupts */
    outb(iobase+EEP_INT_MASK_REG, mask);

#ifdef DEBUG
    if (result != OK)
	kprintf("eep_mcast: mcast setup failed\n");
    else
	kprintf("eep_mcast: mcast setup ok\n");
#endif
    return result;
}


/*-------------------------------------------------------------------------
 * wait_for_exec - wait for the 595's execution to stop (become idle)
 *-------------------------------------------------------------------------
 */
int wait_for_exec(iobase)
u_short iobase;
{
    int i = 0;
    u_char status;

    while (i++ != 1000) {
	status = inb(iobase+EEP_STATUS_REG);
	if (status & EEP_EXEC_ACTIVE) {
	    DELAY(100);
	}
	else
	    return TRUE;
    }
    return FALSE;
}
