#ifndef _ARP_H_
#define _ARP_H_

#include <netif.h>
#include <ip.h>

#define ARPSTK          512
#define ARPPRIO         20
#define ARPNAME         "arp"

#define ARP_MAXENTRIES  50
#define ARP_MAXWAITING  30

#define ARP_MAXADDRLEN  6

/* ARP TYPES DEFINITION */
#define ARP_HTYPE_ETHER 1
#define ARP_PTYPE_IP    0x800

/* return codes for arplookup */
#define ARP_HIT         0
#define ARP_MISS        1
#define ARP_FAIL        2

/* status codes for an arp entry */
#define ARP_STAT_REQUEST        0
#define ARP_STAT_VALID          1
#define ARP_STAT_REVALID        2

#define ARP_CTRL_QSIZE   10
#define ARP_CMDTIMER     0
#define ARP_CMDPKTIN     1
#define ARP_CMDPKTOUT    2

#define ARP_REQUEST     1
#define ARP_REPLY       2

/* ARP timeouts */
#define ARP_TIMER_QUANTUM       500     /* 0.5 seconds */
#define ARP_TIMER_REVALID       2400    /* 20 minutes  */
#define ARP_TIMER_REV_DELTA     120     /* 1 minute    */
#define ARP_TIMER_RETRY         1       /* 0.5 seconds */
#define ARP_NRETRIES            3

struct arp_packet {
        u_short ar_htype;
        u_short ar_ptype;
        u_char  ar_hlen;
        u_char  ar_plen;
        u_short ar_op;
        u_char  ar_data;
};

struct arp_pinfo {
        u_char *ari_hwaddr;
        int     ari_destq;
        struct ep *ari_pep;
        struct arp_pinfo *ari_next;
};

struct arp_entry {
        u_char are_stat;
        u_char are_revalid;
        IPaddr are_ip;
        u_int  are_iface;
        u_char are_hwaddr[ARP_MAXADDRLEN];
        struct arp_pinfo *are_pq;
        struct arp_pinfo *are_pq_tail;
        struct arp_entry *are_next;

        u_int  are_timer;
        u_int  are_retry;
};

struct arp_control {
        int arc_sem;
        int arc_entrypool;
        int arc_pinfopool;
        int arc_ctrlq;
        int arc_pinq;
        int arc_poutq;
        int arc_timerq;
        struct arp_entry *arc_tbl;
};

extern struct arp_control Arp;

PROCESS arp ();
void arpinit ();
void arptimer ();
void arprecv (struct ep *);
void arpin (struct ep *);
void arpout (struct arp_entry*);
int arplookup (IPaddr, u_int, u_char*, struct ep*, u_int);
COMMAND arpshell (int, int, int, int, char **);

#endif /* _ARP_H_ */
