/* ether.h */

#ifndef _ETHER_H_
#define _ETHER_H_

#include <ip.h>

/* Ethernet definitions and constants */

#define	EP_MAXMULTI	  10	/* multicast address table size		*/

#define	EP_MINLEN	  60	/* minimum packet length		*/
#define	EP_DLEN		1500	/* length of data field ep		*/
#define	EP_HLEN		  14	/* size of Ethernet header		*/
#define	EP_ALEN		   6	/* number of octets in physical address	*/
#define	EP_CRC		   4	/* ether CRC (trailer)			*/
#define	EP_MAXLEN	(EP_HLEN+EP_DLEN)
typedef	unsigned char	Eaddr[EP_ALEN]; /* a physical Ethernet address	*/
#define	EP_RETRY	3	/* number of times to retry xmit errors	*/
#define	EP_BRC	"\377\377\377\377\377\377"/* Ethernet broadcast address	*/
#define EP_RTO 		300	/* time out in seconds for reads	*/

#define	EPT_LOOP	0x0060		/* type: Loopback		*/
#define	EPT_PUP		0x0200		/* type: Xerox PUP		*/
#define	EPT_IP		0x0800		/* type: Internet Protocol	*/
#define	EPT_ARP		0x0806		/* type: ARP			*/
#define	EPT_RARP	0x8035		/* type: Reverse ARP		*/

#define EPT_OTHERMASK   0xF000          /* mask for othernet ether type */

struct	eh {			/* ethernet header			*/
	Eaddr	eh_dst;		/* destination host address		*/
	Eaddr	eh_src;		/* source host address			*/
	unsigned short	eh_type;/* Ethernet packet type (see below)	*/
};

struct  ehx {                   /* Xinu extended header                 */
	short   ehx_inif;       /* originating interface number         */
	short   ehx_outif;      /* destination interface number         */
	IPaddr  ehx_nexthop;    /* Next IP hop                          */
	short   ehx_len;        /* length of the packet                 */
	struct ep *ehx_next;	/* next packet in fragmentation cache	*/
        unsigned short ehx_flags;
};

struct  ep      {               /* complete structure of Ethernet packet*/
	struct  ehx ep_ehx;     /* Xinu extended header                 */
	struct  eh ep_eh;       /* the ethernet header                  */
	char    ep_data[EP_DLEN];       /* data in the packet           */
};

/* these allow us to pretend it's all one big happy structure */
#define ep_inif		ep_ehx.ehx_inif
#define ep_outif	ep_ehx.ehx_outif
#define ep_nexthop	ep_ehx.ehx_nexthop
#define ep_len		ep_ehx.ehx_len
#define ep_flags        ep_ehx.ehx_flags
#define ep_nhop         ep_ehx.ehx_nhop
#define ep_next         ep_ehx.ehx_next
#define	ep_dst		ep_eh.eh_dst
#define	ep_src		ep_eh.eh_src
#define	ep_type		ep_eh.eh_type

/* Extended header flags */

#define EPF_VERIFIED	0x0001
#define EPF_ROUTED      0x0002
#define EPF_ARPDONE	0x0004
#define EPF_OPTSDONE	0x0008
#define EPF_PROTOUN	0x0010
#define EPF_DESTUN	0x0020
#define EPF_REDIRECT	0x0040
#define EPF_SRCROUTE	0x0080
#define EPF_BCAST	0x0100

#define	ETOUTQSZ	32	
#define	ETINQSZ		32	

/* ethernet function codes */

#define	EPC_PROMON	1		/* turn on promiscuous mode	*/
#define	EPC_PROMOFF	2		/* turn off promiscuous mode	*/
#define	EPC_MADD	3		/* add multicast address	*/
#define	EPC_MDEL	4		/* delete multicast address	*/

struct ethdev_comm {
    struct devsw	*edc_pdev;
    Eaddr		 edc_paddr;	/* hardware address		*/
    Eaddr		 edc_bcast;
    char		*edc_descr;
    char		 edc_aka[8];
    int		 	 edc_outq;
    int			 edc_inq;	/* frame input queue		*/
    int			 edc_rpid;	/* process trying to read,if any*/
    int     		 edc_rsem;	/* semaphore to protect readers */
    int		 	 edc_ifnum;
#ifdef XPA
    unsigned char        edc_xpa;	/* on/off flag for XPA		*/
    int		         (*edc_xpafunc)();/* user-defined function	*/
#endif
    int			 (*edc_rawrite)();/* rawrite for Othernets	*/
    unsigned char	 edc_proms;	/* promiscuous mode flag	*/
    unsigned char	 edc_mcset;	/* need multicast set flag	*/
    int		 	 edc_mcc;	/* # valid mca's		*/
    Eaddr		 edc_mca[EP_MAXMULTI]; /* multicast addr. table	*/
};
#define ed_pdev		eddev_comm.edc_pdev
#define ed_paddr	eddev_comm.edc_paddr
#define ed_bcast	eddev_comm.edc_bcast
#define ed_descr	eddev_comm.edc_descr
#define ed_aka		eddev_comm.edc_aka
#define ed_outq		eddev_comm.edc_outq
#define ed_inq		eddev_comm.edc_inq
#define ed_rpid		eddev_comm.edc_rpid
#define ed_rsem		eddev_comm.edc_rsem
#define ed_ifnum	eddev_comm.edc_ifnum
#define ed_xpa		eddev_comm.edc_xpa
#define ed_xpafunc	eddev_comm.edc_xpafunc
#define ed_rawrite	eddev_comm.edc_rawrite
#define ed_proms	eddev_comm.edc_proms
#define ed_mcset	eddev_comm.edc_mcset
#define ed_mcc		eddev_comm.edc_mcc
#define ed_mca		eddev_comm.edc_mca

/* Packet Byte Order Constants (set means host order) */

#define EPO_NET         1       /* network layer                */
#define EPO_IP          2       /* level 1 protocols...         */
#define EPO_ARP         2
#define EPO_RARP        2
#define EPO_ICMP        4       /* level 2 protocols            */
#define EPO_IGMP        4
#define EPO_TCP         4
#define EPO_UDP         4
#define EPO_OSPF        4
#define EPO_DNS         5       /* application protocols        */


/*
 * The structure is for EtherExpress PRO/10 port I/O
 */
struct ethdev {
    struct ethdev_comm	eddev_comm;	/* common part 			*/
    unsigned short	ed_rx_start;	/* start of the RX chain	*/
    unsigned short	ed_tx_start;	/* start of the TX chain	*/
    unsigned short	ed_tx_end;	/* end of the TX chain		*/
    unsigned short	ed_tx_chain_bit;/* position to set chain bit	*/
    unsigned short	ed_tx_chain_cnt;/* chain count			*/
    unsigned long	ed_mem_size;	/* on-board memory size		*/
    unsigned short	ed_txbuf_size;	/* TX buffer size		*/
    unsigned short	ed_rxbuf_size;	/* RX buffer size		*/
};

extern struct ethdev     eep[];

#endif
