#ifndef _ICMP_H_
#define _ICMP_H_

#define ICMPT_ECHORP	0
#define ICMPT_DESTUR	3
#define ICMPT_SRCQ	4
#define ICMPT_REDIRECT	5
#define ICMPT_ECHORQ	8
#define ICMPT_RAD	9
#define ICMPT_RSEL	10
#define ICMPT_TIMEX	11
#define ICMPT_PARAM	12
#define ICMPT_TSRQ	13
#define ICMPT_TSRP	14
#define ICMPT_MASKRQ	17
#define ICMPT_MASKRP	18
#define ICMPT_TRCRT	30

#define ICMPC_UNUSED	0

#define ICMP_ERROR(type)	(!((type) == ICMPT_ECHORP \
	|| (type) == ICMPT_ECHORQ || (type) == ICMPT_RAD \
	|| (type) == ICMPT_RSEL || (type) == ICMPT_TSRQ \
	|| (type) == ICMPT_TSRP || (type) == ICMPT_MASKRQ \
	|| (type) == ICMPT_MASKRP || (type) == ICMPT_TRCRT))

/* Destination unreachable */
#define ICMPC_NETUR	0	/* net unreachable			*/
#define ICMPC_HOSTUR	1	/* host unreachable			*/
#define ICMPC_PROTOUR	2	/* protocol unreachable			*/
#define ICMPC_PORTUR	3	/* port unreachable			*/
#define ICMPC_FRAGN	4	/* fragmentation needed and DF set	*/
#define ICMPC_SRCRT	5	/* source route failed			*/
#define	ICMPC_DSTNET	6	/* destination network unknown		*/
#define ICMPC_DSTHOST	7	/* destination host unknown		*/

/* Redirect datagrams for ... */
#define ICMPC_RDNET	0	/* the Network				*/
#define ICMPC_RDHOST	1	/* the Host 				*/
#define ICMPC_RDTOSN	2	/* the Type of Service and Network	*/
#define ICMPC_RDTOSH	3	/* the Type of Service and Host		*/

/* Time exceeded */
#define ICMPC_TTL	0	/* TTL exceeded in transit		*/
#define ICMPC_FRAG	1	/* fragment reassembly time exceeded	*/

/* Parameter problem */
#define ICMPC_POINT	0	/* pointer indicates error		*/
#define ICMPC_MISSOP	1	/* missing required option		*/
#define ICMPC_BADLEN	2	/* bad length				*/

#define ICMPT_ANY	(~0)
#define ICMPC_ANY	(~0)
#define ICMPID_ANY	(~0)

#define ICMP_HLEN	8

#define ICB_FREE	0
#define ICB_OPEN	1

#define ICB_NMQ		16

#define ICMP_TIMEOUT	((u_int)(-2))

#define ICMP_REDIRECT_TIMEOUT	3600	/* 1 hour */


struct icmp {
	u_char		icmp_type;
	u_char		icmp_code;
	u_short		icmp_cksum;
	u_int		icmp_misc;
	u_char		icmp_data[1];
};

struct icmpconn {
	u_int icmp_type;
	u_int icmp_code;
	u_int icmp_id;
	IPaddr icmp_raddr;
};

struct icb {
	int icb_state;
	struct icmpconn icb_conn;
	int icb_mq;
	int icb_dvnum;
	int icb_cookie;
};

struct icmpcontrol {
	int icmpdev;
	int icmpmutex;
};

extern struct icb icbtab[];
extern struct icmpcontrol Icmp;

int icmpgetproto (struct ep *pep);
int icmpconn_eq (struct icmpconn *a, struct icmpconn *b);
u_short icmpcksum (struct ip *pip, struct icmp *picmp);
int icmpechos (struct ep *pep, struct ip *pip, struct icmp *picmp);
void icmpredirect (struct ep *pep, struct ip *pip, struct icmp *picmp);
int icmpreply (int type, int code, u_int misc, struct ep *epep);

PROCESS icmp ();

int icmpservers();

#endif
