#ifndef _IP_H_
#define _IP_H_

typedef	unsigned long IPaddr;	/*  internet address			*/

#include <ether.h>

#define IP_HLEN(pip)	((pip)->ip_hlen << 2)
#define IP_MINHLEN	20

#define IP_DEFTTL	255

#define IPADDR_BCAST	0xffffffff
#define IPADDR_ANY	0x0

#define IPP_ICMP	1
#define IPP_TCP		6
#define IPP_UDP		17

#define IPF_DF		0x4000
#define IPF_MF		0x2000
#define IP_OFFMASK	0x1fff

#define IP_OFF(pip)	((net2hs((pip)->ip_offset) & IP_OFFMASK) << 3)

#define IPC_MJOIN       0
#define IPC_MLEAVE      1

#define IP_ISMULTI(x)	((x & 0x000000f0) == 0x000000e0)

#define IP_SMLBUFSZ	MAXNETBUF
#define IP_NSMLBUF	NETBUFS
#define IP_MEDBUFSZ	(8192 + sizeof (struct eh) + sizeof (struct ehx))
#define IP_NMEDBUF	8
#define IP_LRGBUFSZ	(65536 + sizeof (struct eh) + sizeof (struct ehx))
#define IP_NLRGBUF	4

#define IPFC_N		10
#define IPFC_FRAGMAX	100	/* max frags allowed in FC */
#define IPFC_TIMEOUT	60000	/* 60 seconds */

#define IP_FQSIZE	10
#define IP_SQSIZE	10

struct ip {
#if BYTE_ORDER == BIG_ENDIAN
	unsigned int	ip_ver:4;
	unsigned int	ip_hlen:4;
#else
	unsigned int	ip_hlen:4;
	unsigned int	ip_ver:4;
#endif
	u_char		ip_tos;
	u_short		ip_len;
	u_short		ip_id;
	u_short		ip_offset;
	u_char		ip_ttl;
	u_char		ip_proto;
	u_short		ip_cksum;
	IPaddr		ip_src;
	IPaddr		ip_dst;
	u_char		ip_data[1];
};

struct ipdev {
	int ipd_proto;
	int ipd_mq;
	int ipd_dvnum;
	u_char ipd_tos;
	u_char ipd_ttl;
};

struct ipcontrol {
	int fastipmq;
	int slowipmq;
	int forward;
	u_short fragid;
	int devmutex;
	int mbpool;
	int lbpool;
	int nfrags;
};


struct ipfc {
	struct ep *head;
	struct ep *hole;   /* ptr to frag before hole, NULL if at beginning */
	int firstts;	   /* ts of first frag 				    */
	int rcntts;	   /* ts of most recent fragment		    */
};


extern struct ipdev ipdevtab[];
extern struct ipcontrol Ip;
extern struct ipfc ipfctab[];

char *iptochar (IPaddr, char*);
IPaddr chartoip (char*);

#define ipalloc(ipdev, len, src, dst) _ipalloc (ipdev, len, src, dst, FALSE)
#define nbipalloc(ipdev, len, src, dst) _ipalloc (ipdev, len, src, dst, TRUE)
struct ep *_ipalloc (int ipdev, int len, IPaddr src, IPaddr dst, int block);
void iprecv (struct ep *pep);
int iplocal (int nif, IPaddr addr);
inline int ipttl (struct ip *pip);
int ipsend (struct ep *pep);
int iplookup(struct ep *pep);
int iproute (struct ep *pep, struct ip *pip);
void ipdeliver (struct ep *pep, struct ip *pip);
inline struct ep * _ipgetbuf (int buflen, int block);
void ipreassemble (struct ep *pep, struct ip *pip);
int ipprintfc (int stdout);
void ipfctimeout ();

PROCESS fastip ();
PROCESS slowip ();

#endif
