#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <mq.h>
#include <stdio.h>

/* #define DEBUG */

int icmpconn_eq (struct icmpconn *a, struct icmpconn *b) {
	return (a->icmp_type == b->icmp_type || a->icmp_type == ICMPT_ANY
			|| b->icmp_type == ICMPT_ANY)
		&& (a->icmp_code == b->icmp_code || a->icmp_code == ICMPC_ANY
			|| b->icmp_code == ICMPC_ANY)
		&& (a->icmp_id == b->icmp_id || a->icmp_id == ICMPID_ANY
			|| b->icmp_id == ICMPID_ANY)
		&& (a->icmp_raddr == b->icmp_raddr
			|| a->icmp_raddr == IPADDR_ANY
			|| b->icmp_raddr == IPADDR_ANY);
}

PROCESS icmp ()
{
	struct ep *pep;
	struct ip *pip;
	struct icmp *picmp;
	struct icmpconn conn;
	int i;

	while (1) {
		if (read (Icmp.icmpdev, &pep, sizeof(pep)) == SYSERR) {
			close (Icmp.icmpdev);
			return SYSERR;
		}
		pip = (struct ip *)pep->ep_data;
		if (pip->ip_proto != IPP_ICMP) {
			kprintf ("ICMP: wrong packet type %d\n", pip->ip_proto);
			freebuf (pep);
			continue;
		}
		picmp = (struct icmp *)((char *)pip + IP_HLEN(pip));

		if (net2hs (pip->ip_len) < IP_HLEN(pip) + ICMP_HLEN
		    || icmpcksum (pip, picmp)) {
#ifdef DEBUG
			kprintf ("ICMP cksum failed\n");
#endif
			freebuf (pep);
			continue;
		}

		if (picmp->icmp_type == ICMPT_ECHORQ) {
			icmpechos (pep, pip, picmp); /* Frees pep */
			continue;
		}
		if (picmp->icmp_type == ICMPT_REDIRECT) {
			icmpredirect (pep, pip, picmp); /* Frees pep */
			continue;
		}

		wait (Icmp.icmpmutex);
		for (i = 0; i < Nicmp; i++) {
			conn.icmp_type = picmp->icmp_type;
			conn.icmp_code = picmp->icmp_code;
			conn.icmp_id = net2hl (picmp->icmp_misc) >> 16;
			conn.icmp_raddr = pip->ip_src;
			if (icbtab[i].icb_state == ICB_OPEN
			    && icmpconn_eq (&(icbtab[i].icb_conn), &conn)) {
			    	if (mqsend (icbtab[i].icb_mq, (int)pep) == SYSERR)
					freebuf (pep);
				break;
			}
		}
		if (i == Nicmp)
			freebuf (pep);
#ifdef DEBUG
		else kprintf ("icmp packet delivered to %d\n", i);
#endif
		signal (Icmp.icmpmutex);
	}

	return OK;
}
