#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ipopt.h>
#include <stdio.h>

/* #define DEBUG */

/* TODO Source Route */
static int ping_opts (struct ep *pep, struct ip *pip, IPaddr myip)
{
	int i = 0, j, olen, type, ptr;
	IPaddr *a, *b, tmp;
	while (i < IP_HLEN(pip) - IP_MINHLEN) {
		olen = pip->ip_data[i+1];
		type = pip->ip_data[i];
		if (type == IPOPT_EOL) {
			break;
		} else if (type == IPOPT_NOP) {
			olen = 1;
		} else if (type == IPOPT_RR) {
			/* do nothing */
		} else if (type == IPOPT_TS) {
			/* do nothing */
		} else if (type == IPOPT_LSRR || type == IPOPT_SSRR) {
			ptr = pip->ip_data[i+2];
			pip->ip_data[i+2] = 4;
			a = (IPaddr *) &(pip->ip_data[i+3]);
			b = (IPaddr *) ((int) a + ptr - sizeof(IPaddr));
			b--;
			tmp = pip->ip_dst;
			pip->ip_dst = *b;
			*b = tmp;
			b--;
			for (; a < b; a++, b--) {
				tmp = *a;
				*a = *b;
				*b = tmp;
			}
		} else {
			/* stamp out everything else */
			for (j = i; j < i + olen; j++)
				pip->ip_data[j] = 0x1;
		}
		i += olen;
	}

	return OK;
}

/* icmpechos will free pep */
int icmpechos (struct ep *pep, struct ip *pip, struct icmp *picmp)
{
	IPaddr tmp;

	pep->ep_inif = NI_LOCAL;
	pep->ep_outif = NI_LOCAL;
	pep->ep_flags = 0;

	pip = (struct ip *)pep->ep_data;
	picmp = (struct icmp *)((char *)pip + IP_HLEN(pip));

	picmp->icmp_type = ICMPT_ECHORP;
	picmp->icmp_cksum = 0;
	picmp->icmp_cksum = icmpcksum (pip, picmp);
#ifdef DEBUG
	kprintf ("Received ping. seq=%d\n", net2hl(picmp->icmp_misc) & 0xffff);
#endif

	tmp = pip->ip_src;
	pip->ip_src = pip->ip_dst;
	pip->ip_dst = tmp;
	pip->ip_ttl = IP_DEFTTL; /* XXX */
	pip->ip_id = Ip.fragid++;
	if (IP_HLEN(pip) != IP_MINHLEN) {
#ifdef DEBUG
		kprintf("Handling IP options, hlen=%d\n", IP_HLEN(pip));
#endif
		if (ping_opts (pep, pip, pip->ip_src) == SYSERR) {
			freebuf (pep);
			return SYSERR;
		}
	}

	/* cksum computed in ipsend */
	return ipsend (pep);
}
