#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ipif.h>
#include <stdio.h>

/* Send an ICMP reply to an ip packet
 * misc must be in network byte order
 * function returns if packet was an ICMP message, a fragment,
 *	src of 0.0.0.0 or dst was broadcast
 */
int icmpreply (int type, int code, u_int misc, struct ep *epep)
{
	struct ip *epip = (struct ip *)epep->ep_data;
	struct ep *pep;
	struct ip *pip;
	struct icmp *picmp;
	int datalen;

	/* TODO: check for multicast or Class E */
	if ((epip->ip_proto == IPP_ICMP
		&& ICMP_ERROR(*((u_char*) epip + IP_HLEN(epip))))
			|| epip->ip_dst == IPADDR_BCAST
			|| epip->ip_src == IPADDR_BCAST
			|| IP_ISMULTI(epip->ip_dst)
			|| IP_ISMULTI(epip->ip_src)
			|| ipif[epep->ep_inif].ipif_bcast == epip->ip_dst
			|| ipif[epep->ep_inif].ipif_bcast == epip->ip_src
			|| strncmp ((char *)epep->ep_eh.eh_dst, EP_BRC, EP_ALEN) == 0
			|| net2hs (epip->ip_offset) & IP_OFFMASK
			|| epip->ip_src == 0)
		return OK;

	/* only send ip header if bad parameters */
	if (type == ICMPT_PARAM)
		datalen = IP_HLEN(epip);
	else
		datalen = min(128, net2hs(epip->ip_len));

	if ((pep = ipalloc (Icmp.icmpdev, ICMP_HLEN + datalen,
			IPADDR_ANY, epip->ip_src)) == NULL)
		return OK;
	pip = (struct ip *)pep->ep_data;
	picmp = (struct icmp*)((char *)pip + IP_HLEN(pip));
	picmp->icmp_type = type;
	picmp->icmp_code = code;
	picmp->icmp_cksum = 0;
	picmp->icmp_misc = misc;
	blkcopy (picmp->icmp_data, epip, datalen);
	picmp->icmp_cksum = icmpcksum (pip, picmp);
	ipsend (pep);
	
	return OK;
}
