#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <mq.h>
#include <ipif.h>
#include <iproute.h>
#include <stdio.h>

PROCESS fastip ()
{
	struct ep *pep;
	struct ip *pip;

	while (1) {
		pep = (struct ep *)mqrecv (Ip.fastipmq);
		pip = (struct ip *)pep->ep_data;
		if (!(pep->ep_flags & EPF_VERIFIED)) {
			if (pep->ep_len < sizeof(struct eh) + IP_MINHLEN
			    || IP_HLEN(pip) < IP_MINHLEN
			    || net2hs (pip->ip_len) < IP_HLEN(pip)
			    || pep->ep_len < sizeof(struct eh)
			                     + net2hs (pip->ip_len)) {
				freebuf (pep);
				continue;
			}
			if (pep->ep_inif != NI_LOCAL
			    && cksum (pip, IP_HLEN(pip))) {
				freebuf (pep);
				continue;
			}
			pep->ep_flags |= EPF_VERIFIED;
		}

		/* Packets returning from ARP have already been processed */
		if (pep->ep_flags & EPF_ARPDONE) {
			ni_out (pep);
			continue;
		}

		if (IP_HLEN(pip) != IP_MINHLEN
		    && !(pep->ep_flags & EPF_OPTSDONE)) {
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			continue;
		}

		if (!(pep->ep_flags & EPF_ROUTED)) {
			if (iproute (pep, pip) == SYSERR)
				continue;
		}

		if (pep->ep_outif != NI_LOCAL
		    && net2hs (pip->ip_len) > nif[pep->ep_outif].ni_mtu) {
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			continue;
		}

		if (pep->ep_outif == NI_LOCAL
		    && net2hs (pip->ip_offset) & (IP_OFFMASK | IPF_MF)) {
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			continue;
		}

		ipdeliver (pep, pip);
	}
}
