#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <iproute.h>
#include <ipif.h>
#include <stdio.h>
#include <io.h>

#define _GETBUF(block) (block ? (getbuf) : (nbgetbuf))

inline struct ep * _ipgetbuf (int buflen, int block)
{
	if (buflen <= IP_SMLBUFSZ)
		return (struct ep *)_GETBUF(block)(Net.netpool);
	else if (buflen <= IP_MEDBUFSZ)
		return (struct ep *)_GETBUF(block)(Ip.mbpool);
	else if (buflen <= IP_LRGBUFSZ)
		return (struct ep *)_GETBUF(block)(Ip.lbpool);
	else
		return NULL;
}

struct ep *_ipalloc (int ipdev, int len, IPaddr src, IPaddr dst, int block)
{
	struct devsw *pdev;
	struct ipdev *pipd;
	struct ep *pep;
	struct ip *pip;
	struct route *prt;

	if (isbaddev (ipdev) || (pdev = &devtab[ipdev])->dvminor > Nip
	    || ipdev != (pipd = &ipdevtab[pdev->dvminor])->ipd_dvnum)
		return NULL;

	pep = _ipgetbuf (len + IP_MINHLEN + sizeof(struct eh)
			+ sizeof(struct ehx), block);
	if (pep == (struct ep *)SYSERR || pep == NULL)
		return NULL;

	pip = (struct ip *)pep->ep_data;
	pip->ip_ver = 4;
	pip->ip_hlen = IP_MINHLEN >> 2;
	pep->ep_inif = NI_LOCAL;
	pep->ep_outif = NI_LOCAL;
	pep->ep_len = IP_HLEN(pip) + len + sizeof(pep->ep_eh);
	pep->ep_type = EPT_IP;
	pep->ep_flags = 0;
	pip->ip_tos = pipd->ipd_tos;
	pip->ip_len = hs2net (IP_HLEN(pip) + len);
	pip->ip_id = Ip.fragid++;
	pip->ip_offset = 0;
	pip->ip_ttl = pipd->ipd_ttl;
	pip->ip_proto = pipd->ipd_proto;

	pip->ip_src = src;
	pip->ip_dst = dst;

	if (dst == IPADDR_ANY)
		return pep;

	if (src == IPADDR_ANY) {
		if ((prt = rtget (dst)) == NULL) {
			/* Make a guess */
			pip->ip_src = ipif[NI_PRIMARY].ipif_addr;
		} else {
			pip->ip_src = ipif[prt->rt_ifnum].ipif_addr;
			rtfree (prt);
		}
	}

	return pep;
}
