#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <bufpool.h>
#include <ipif.h>
#include <mq.h>

void ipdeliver (struct ep *pep, struct ip *pip)
{
	int i, ret, proto;

	if (pep->ep_outif == NI_LOCAL) {
		proto = pip->ip_proto;
		if (proto == IPP_ICMP) {
			if ((proto = icmpgetproto (pep)) == SYSERR) {
				freebuf (pep);
				return;
			}
		}
		for (i = 0; i < Nip; i++) {
			if (ipdevtab[i].ipd_proto == proto) {
				if (mqsend (ipdevtab[i].ipd_mq, (int)pep) == SYSERR)
					freebuf (pep);
				break;
			}
		}
		if (i == Nip) {
			pep->ep_flags |= EPF_PROTOUN;
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			return;
		}
	} else {
		/* Check for bogus broadcast */
		if (pep->ep_inif != NI_LOCAL
		    && pip->ip_dst == ipif[pep->ep_outif].ipif_bcast) {
			freebuf (pep);
			return;
		}

		/* Check for expired TTL */
		if (pep->ep_inif != NI_LOCAL && ipttl(pip) == 0) {
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			return;
		}

		if (!(pep->ep_flags & (EPF_REDIRECT | EPF_SRCROUTE))
		    && pep->ep_inif != NI_LOCAL
		    && pep->ep_inif == pep->ep_outif
		    && (pip->ip_src & ipif[pep->ep_outif].ipif_mask)
				== ipif[pep->ep_outif].ipif_net) {
			pep->ep_flags |= EPF_REDIRECT;
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			return;
		}

		/* Try an ARP lookup */
		pep->ep_flags |= EPF_ARPDONE;
		ret = arplookup (pep->ep_nexthop, pep->ep_outif,
				 pep->ep_eh.eh_dst, pep, Ip.fastipmq);
		if (ret == ARP_FAIL) {
			freebuf (pep);
			return;
		} else if (ret == ARP_MISS) {
			/* ARP will take it from here */
			return;
		}
		/* ARP lookup succeeded immediately */
		ni_out (pep);
	}
}
