#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ip.h>
#include <ipopt.h>
#include <stdio.h>

int ipopt(struct ep *pep, struct ip *pip) {
        u_char *prr   = NULL;    /* MUST - record route pointer    */
        u_char *pts   = NULL;    /* MUST - timestamp               */
        u_char *psec  = NULL;    /* OPTIONAL - security option     */
        u_char *plsrr = NULL;    /* MUST - loose source and record */
        u_char *pssrr = NULL;    /* MUST - strict source and rcord */
        u_char *psid  = NULL;    /* NO PROCESSING - stream id      */
        u_char *perr  = NULL;

        u_char *pop  = ((u_char *) pip) + IP_MINHLEN;
        u_char *peop = ((u_char *) pip) + IP_HLEN(pip);
        
        u_char bogus = FALSE;

        while (pop<peop) {
                switch(*pop) {
                case IPOPT_EOL:
                        pop=peop;
                        break;
                case IPOPT_NOP:
                        pop++;
                        break;
                case IPOPT_RR:
                        if (prr == NULL) prr = pop; 
                        else { bogus = TRUE; perr = prr; }
                        pop += *(pop+1);
                        break;
                case IPOPT_TS:
                        if (pts == NULL) pts = pop; 
                        else { bogus = TRUE; perr = pts; }
                        pop += *(pop+1);
                        break;
                case IPOPT_SEC:
                        if (psec == NULL) psec = pop; 
                        else { bogus = TRUE; perr = psec; }
                        pop += *(pop+1);
                        break;
                case IPOPT_LSRR:
                        if (plsrr == NULL) plsrr = pop;
                        else { bogus = TRUE; perr = plsrr; }
                        pop += *(pop+1);
                        break;
                case IPOPT_SSRR:
                        if (pssrr == NULL) pssrr = pop;
                        else { bogus = TRUE; perr = pssrr; }
                        pop += *(pop+1);
                        break;
                case IPOPT_SID:
                        if (psid == NULL) psid = pop;
                        else { bogus = TRUE; perr = psid; }
                        pop += *(pop+1);
                        break;
                default: pop++;
                }
                
        }

        if (bogus) {
                icmpreply(ICMPT_PARAM, ICMPC_POINT,
                        ((int) perr - (int) pip), pep);
                return SYSERR;
        }

        if (plsrr && pssrr) {
                icmpreply(ICMPT_PARAM, ICMPC_POINT,
                        ((int) plsrr - (int) pip), pep);

                return SYSERR;
        }

        if (plsrr) if(optlsrr(pep, pip, plsrr)==SYSERR) return SYSERR;
        if (pssrr) if(optssrr(pep, pip, pssrr)==SYSERR) return SYSERR;
        if (prr)   if(optrr(pep, pip, prr)==SYSERR) return SYSERR;
        if (pts)   if(optts(pep, pip, pts, IPADDR_ANY)==SYSERR) return SYSERR;

        pip->ip_cksum = 0; 
        pip->ip_cksum = (cksum (pip, IP_HLEN(pip)));

	pep->ep_flags |= EPF_OPTSDONE;

        return OK;
}

