#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <bufpool.h>
#include <ipif.h>
#include <mq.h>

int iproute (struct ep *pep, struct ip *pip)
{
	if (pep->ep_inif == NI_LOCAL && (pip->ip_dst == IPADDR_BCAST
					 || IP_ISMULTI(pip->ip_dst))) {
		/* limited broadcast, multicast -- no route lookup */
		if (pep->ep_outif == NI_LOCAL) {
			freebuf (pep);
			return SYSERR;
		}
		pep->ep_nexthop = pip->ip_dst;
	} else {
		/* If there is no configured address on the incoming
		 * interface, deliver the packet locally */
		if (ipif[pep->ep_inif].ipif_addr == IPADDR_ANY)
			pep->ep_outif = NI_LOCAL;
		else if ((pip->ip_dst == IPADDR_BCAST
			  && pep->ep_inif != NI_LOCAL)
			 || (pip->ip_dst == ipif[pep->ep_inif].ipif_bcast)) {
			pep->ep_flags |= EPF_BCAST;
			pep->ep_outif = NI_LOCAL;
		} else if (iplookup (pep) == SYSERR) {
			pep->ep_flags = EPF_DESTUN;
			if (mqsend (Ip.slowipmq, (int)pep) == SYSERR)
				freebuf (pep);
			return SYSERR;
		}
	}
	return OK;
}
