#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ipif.h>
#include <iproute.h>
#include <mq.h>
#include <io.h>
#include <stdio.h>

int ipsend (struct ep *pep)
{
	struct ip *pip = (struct ip *)pep->ep_data;;
	struct route *prt;

	if (pip->ip_dst == IPADDR_ANY) {
		freebuf (pep);
		return SYSERR;
	}
	if (pip->ip_src == IPADDR_ANY) {
		if ((prt = rtget (pip->ip_src)) == NULL) {
			/* Broadcast packets get the requested interface,
			 * so that bootp can function.  Other packets get
			 * NI_PRIMARY ... maybe we should have a default
			 * interface of some sort? */
			if (pip->ip_dst == IPADDR_BCAST)
				pip->ip_src = ipif[pep->ep_outif].ipif_addr;
			else
				pip->ip_src = ipif[NI_PRIMARY].ipif_addr;
		} else {
			pip->ip_src = ipif[prt->rt_ifnum].ipif_addr;
			rtfree (prt);
		}
	}

	pip->ip_cksum = 0;
	pip->ip_cksum = (cksum (pip, IP_HLEN(pip)));

	/* Failing to send does not return SYSERR, as this is the same
	 * as any other packet drop. */
	if (mqsend (Ip.fastipmq, (int) pep) == SYSERR)
		freebuf (pep);

	return OK;
}
