#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ipopt.h>
#include <mq.h>
#include <stdio.h>

PROCESS slowip ()
{
	struct ep *pep, *fragep;
	struct ip *pip, *fragip;
	int remaining, len, offset, ipoff, hlen, origoff;

	while (1) {
		pep = (struct ep *)mqrecv (Ip.slowipmq);

		if (pep == NULL) {
			ipfctimeout();
			continue;
		}

		pip = (struct ip *)pep->ep_data;
		if (pep->ep_flags & EPF_REDIRECT) {
			icmpreply (ICMPT_REDIRECT, ICMPC_RDHOST,
				pep->ep_nexthop, pep);
			ipdeliver (pep, pip);
		}

		if (pep->ep_flags & EPF_PROTOUN) {
			icmpreply (ICMPT_DESTUR, ICMPC_PROTOUR, 0, pep);
			freebuf (pep);
			continue;
		}
		if (pep->ep_flags & EPF_DESTUN) {
			icmpreply (ICMPT_DESTUR, ICMPC_HOSTUR, 0, pep);
			freebuf (pep);
			continue;
		}

		if (pip->ip_ttl == 0) {
			icmpreply (ICMPT_TIMEX, ICMPC_TTL, 0, pep);
			freebuf (pep);
			continue;
		}

		if (IP_HLEN(pip) != IP_MINHLEN
		    && !(pep->ep_flags & EPF_OPTSDONE)
		    && ipopt (pep, pip) == SYSERR) {
			freebuf (pep);
			continue;
		}

		if (!(pep->ep_flags & EPF_ROUTED)) {
			if (iproute (pep, pip) == SYSERR)
				continue;
		}

		if (pep->ep_outif != NI_LOCAL
		    && net2hs (pip->ip_len) > nif[pep->ep_outif].ni_mtu) {
			if (net2hs (pip->ip_offset) & IPF_DF) {
				icmpreply (ICMPT_DESTUR, ICMPC_FRAGN, 0, pep);
				freebuf (pep);
				continue;
			}
			/* fragment */
			remaining = net2hs (pip->ip_len) - IP_HLEN(pip);
			offset = IP_HLEN(pip);
			origoff = (net2hs (pip->ip_offset) & IP_OFFMASK) << 3;
			while (remaining) {
				fragep = (struct ep *)getbuf (Net.netpool);
				blkcopy (fragep, pep, sizeof(struct ehx)
					 	      + sizeof(struct eh)
						      + sizeof(struct ip));
				fragip = (struct ip *)fragep->ep_data;
				if (offset == IP_HLEN(pip))
					blkcopy (fragip->ip_data, pip->ip_data,
						 IP_HLEN(pip) - IP_MINHLEN);
				else
					optfrag (pip, fragip);
				len = nif[pep->ep_outif].ni_mtu
				      - IP_HLEN(fragip);
				len = min(len, remaining);
				if (len != remaining)
					len &= 0xfff8;
				remaining -= len;
				hlen = IP_HLEN(fragip);
				fragep->ep_len = sizeof(struct eh)+ hlen + len;
				fragip->ip_len = hs2net (len + hlen);
				ipoff = (origoff + offset - IP_HLEN(pip)) >> 3;
				if (remaining || net2hs (pip->ip_offset) & IPF_MF)
					ipoff |= IPF_MF;
				fragip->ip_offset = hs2net (ipoff);
				fragip->ip_cksum = 0;
				fragip->ip_cksum = cksum ((char *)fragip, hlen);
				blkcopy ((char *)fragip + hlen,
					 (char *)pip + offset, len);
				offset += len;
				/* sleep1000(1); XXX*/
				ipdeliver (fragep, fragip);
			}
			freebuf (pep);
			continue;
		}

		if (pep->ep_outif == NI_LOCAL
		    && net2hs(pip->ip_offset) & (IP_OFFMASK | IPF_MF)) {
			ipreassemble (pep, pip);
			continue;
		}

		ipdeliver (pep, pip);
	}
}
