#include <conf.h>
#include <kernel.h>
#include <iproute.h>
#include <stdio.h>

void rtprint(int stdout, struct route *prt);
char *ip2str(IPaddr, char *str);

void rtdump(int stdout)
{
        struct route *prt;

        fprintf(stdout, "Xinu lookup IP routing table\n");
        fprintf(stdout, "Destination     Gateway         Genmask          Ifnum Metric    Ttl    Tag  Index Prefix\n");

        wait (Route.ri_wsem);

        for(prt=RT_PHEAD(Route.ri_lookup); prt; prt=RT_PNEXT(Route.ri_lookup, prt))
                rtprint(stdout, prt);

        fprintf(stdout, "\n");
        fprintf(stdout, "Xinu working IP routing table\n");
        fprintf(stdout, "Destination     Gateway         Genmask          Ifnum Metric    Ttl    Tag  Index Prefix\n");

        for(prt=RT_PHEAD(Route.ri_working); prt; prt=RT_PNEXT(Route.ri_working, prt))
                rtprint(stdout, prt);
        signal (Route.ri_wsem);
}

void rtprint(int stdout, struct route *prt) {
               char str[RT_MAXIPSTR];
               fprintf(stdout, "%-15s ", ip2str(prt->rt_net, str));
               fprintf(stdout, "%-15s ", ip2str(prt->rt_gw, str));
               fprintf(stdout, "%-15s ", ip2str(prt->rt_mask, str));
               fprintf(stdout, "%6d %6d %6d %6d %6d %6d\n",
                        prt->rt_ifnum,
                        prt->rt_metric,
                        prt->rt_ttl,
                        prt->rt_tag,
                        prt->rt_iself,
                        prt->rt_iprefix);
}

char *ip2str(IPaddr ip, char *str) {
        sprintf(str, "%d.%d.%d.%d",
                        ((unsigned char *)&ip)[0],
                        ((unsigned char *)&ip)[1],
                        ((unsigned char *)&ip)[2],
                        ((unsigned char *)&ip)[3]);
        return(str);
}

void trieprint(int stdout, struct rttable *prtable) {
        int i;

        unsigned int node, branch, skip, isleaf, index;

        fprintf(stdout, "\n Pointer table: \n");
        for(i=0;i<=prtable->rtb_plast;i++)
                fprintf(stdout, "index=%d pointer=%x\n", i, (unsigned int) prtable->rtb_prefix[i]);

        fprintf(stdout, "\n The trie: \n");
        for(i=0;i<=15;i++) {
                node = prtable->rtb_trie[i];
                branch = RT_BITEXTRACT(node, 0, 5);
                skip = RT_BITEXTRACT(node, 5, 5);
                isleaf = RT_BITEXTRACT(node, 10, 1);
                index = RT_BITEXTRACT(node, 11, 21);
                fprintf(stdout, "index=%02d, value=%08x, branch=%2d, skip=%2d, isleaf=%2d, index=%2d\n",
                        i, prtable->rtb_trie[i],
                        branch, skip, isleaf, index
                        );
        }
}

