#include <conf.h>
#include <kernel.h>
#include <iproute.h>
#include <stdio.h>

#define RTROUTE_ADD             "add"
#define RTROUTE_DEL             "del"
#define RTROUTE_LOOKUP          "lookup"
#define RTROUTE_TRIEPRINT       "trie"

int rtroute (int stdin, int stdout, int stderr,
                int nargs, char *args[])
{
        IPaddr net, mask, gw;
        int ifnum, metric, ttl, tag, notadv;
        IPaddr dest;
        struct route *prt;

        if(nargs==1) {
                rtdump(stdout);
                return OK;
        }

        if(strcmp(args[1], RTROUTE_ADD)==0) {
                if (nargs!=10) {
                        fprintf(stdout, "Usage: route add <net> <mask> <gw> <ifnum> <metric> <ttl> <tag> <notadv>\n");
                        return SYSERR;
                }
                        
                net = chartoip(args[2]);
                mask = chartoip(args[3]);
                gw = chartoip(args[4]);
                sscanf (args[5], "%d", &ifnum);
                sscanf (args[6], "%d", &metric);
                sscanf (args[7], "%d", &ttl);
                sscanf (args[8], "%d", &tag);
                sscanf (args[9], "%d", &notadv);
                rtadd (net, mask, gw, ifnum, metric, ttl, tag, notadv);
                return OK;
        }

        if (strcmp(args[1], RTROUTE_DEL)==0) {
                if (nargs!=4) {
                        fprintf(stdout, "Usage: route del <net> <mask>");
                        return SYSERR;
                }
                net = chartoip(args[2]);
                mask = chartoip(args[3]);
                if (rtdel (net, mask) == SYSERR) {
                        fprintf(stdout, "route: Unable to remove this route\n");
                }
                return(OK);
        }

        if (strcmp (args[1], RTROUTE_LOOKUP)==0) {
                if(nargs!=3) {
                        fprintf(stdout, "Usage: route lookup <destination>\n");
                        return SYSERR;
                }
                dest = chartoip (args[2]);
                prt = rtget (dest);
                if (prt) {
                        fprintf(stdout, "Destination     Gateway         Genmask          Ifnum Metric    Ttl    Tag  Index Prefix\n");
                        rtprint (stdout, prt);
                        rtfree (prt);
                } else
                        fprintf (stdout, "No route to this destination\n");
                return OK;
        }

        if (strcmp (args[1], RTROUTE_TRIEPRINT)==0) {
                wait (Route.ri_wsem);
                trieprint (stdout, Route.ri_working);
                signal (Route.ri_wsem);
                return OK;
        }

        fprintf(stdout, "route: invalid command\n");
        return SYSERR;
}

