#include <conf.h>
#include <kernel.h>
#include <iproute.h>

void rttrie_build_prefix(void);
void rttrie_build_trie(u_int start, u_int count, u_int common, u_int nfree);
u_int rttrie_getskip(u_int start, u_int count, u_int common);
u_int rttrie_getbranch(u_int start, u_int count, u_int common, u_int skip);

void rttrie(void)
{
        rttrie_build_prefix();
        Route.ri_working->rtb_trie[0] = 0;
        Route.ri_working->rtb_nfree = 1;
        if (Route.ri_working->rtb_plast!=-1)
                rttrie_build_trie(0, Route.ri_working->rtb_plast+1, 0, 0);
}

void rttrie_build_prefix(void) {
        struct route *prt;
        struct route *prev;

        prev = NULL;
        Route.ri_working->rtb_plast = -1;
        for (prt=RT_PHEAD(Route.ri_working); prt; prt=RT_PNEXT(Route.ri_working, prt)) {
                if (!prev || prev->rt_net != prt->rt_net)
                        Route.ri_working->rtb_prefix[++Route.ri_working->rtb_plast] =
                                prt;
                prev=prt;
        }
}

void rttrie_build_trie(u_int start, u_int count, u_int common, u_int nfree) {
        u_int skip, branch;
        u_int fchild;
        u_int i, binc;
        u_int last;

        if (count == 1) {
               Route.ri_working->rtb_trie[nfree] = RT_BUILD_LEAF(0, 0, start);
               return;
        }

        skip = rttrie_getskip(start, count, common);
        branch = rttrie_getbranch(start, count, common, skip);
        fchild = Route.ri_working->rtb_nfree; 
        Route.ri_working->rtb_nfree += (1 << branch);
        Route.ri_working->rtb_trie[nfree] = RT_BUILD_NODE(branch, skip, fchild);

        last = start + count - 1;
        for (i=0; i<(1 << branch); i++) {
                binc = 0;
                while(start+binc<=last &&
                      RT_BITEXTRACT(
                        net2hl(Route.ri_working->rtb_prefix[start+binc]->rt_net),
                        common+skip, branch) == i)
                                binc++;
                rttrie_build_trie(start, binc, common+skip+branch, fchild+i);
                start += binc;
        }
}

u_int rttrie_getskip(u_int start, u_int count, u_int common) {
        u_int skip=0;
        u_int result = ~(net2hl(Route.ri_working->rtb_prefix[start]->rt_net) ^
                         net2hl(Route.ri_working->rtb_prefix[start+count-1]->rt_net))
                         << common;

        while ((result & 0x80000000) && skip++<32)
                result = result << 1;
        return (skip);
}

u_int rttrie_getbranch(u_int start, u_int count, u_int common, u_int skip) {
        u_int branch=1;
        u_int isfull=TRUE;
        u_int test, i;

        while (isfull) {
                test=0;
                if((1<<branch)>count) break;
                for(i=start; i<start+count; i++) {
                        if ( ((net2hl(Route.ri_working->rtb_prefix[i]->rt_net) << (common+skip))
                                >> (32 - branch)) == test) test++;
                        if ( test == (1 << branch))
                                break;
                }
                if(test == (1<<branch)) branch++;
                else isfull = FALSE;
        }

        return (branch-1);
}

