#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <stdio.h>
#include <bootp.h>
#include <ipif.h>
#include <page_server.h>

extern int ctr1000;

PROCESS bootp_in ();

PROCESS bootp ()
{
	char buf[sizeof(struct bootp) + UG_HLEN];
	struct udpdgram *pug = (struct udpdgram *)buf;
	struct bootp *pbp = (struct bootp *)pug->ug_data;
	int fd, inpid, xid, secs = 0, ifnum = NI_PRIMARY;

	nif[ifnum].ni_state = NIS_UP;

	if ((fd = open (UDP, (int)LISTENPORT(UDPP_BOOTPC), 0)) == SYSERR) {
		kprintf ("bootp: udp open failed\n");
		return SYSERR;
	}
	if (control (fd, UDPC_CKSUM, FALSE) == SYSERR) {
		kprintf ("bootp: error disabling checksums\n");
	}
	inpid = create ((int *)bootp_in, 512, 40, "bootp_in",
			4, fd, getpid (), (xid = ctr1000), ifnum);
	if (inpid == SYSERR) {
		kprintf ("bootp: process create failed\n");
		close (fd);
		return SYSERR;
	}
	resume(inpid);
	bzero (buf, sizeof(buf));
	pug->ug_ip = IPADDR_BCAST;
	pug->ug_port = UDPP_BOOTPS;
	pug->ug_len = sizeof(struct bootp);
	pug->ug_nif = ifnum;
	pbp->bp_op = BPOP_REQ;
	pbp->bp_htype = nif[ifnum].ni_hwtype;
	pbp->bp_hlen = nif[ifnum].ni_hwa.ha_len;
	pbp->bp_xid = hl2net(xid); /* byte swap unnecessary, but consistent */
	blkcopy (pbp->bp_chaddr, nif[ifnum].ni_hwa.ha_addr, pbp->bp_hlen);
	blkcopy (pbp->bp_vend, BOOTP_MAGIC, strlen (BOOTP_MAGIC));
	pbp->bp_vend[sizeof(BOOTP_MAGIC) - 1] = BOOTPO_PAD;
	pbp->bp_vend[sizeof(BOOTP_MAGIC)] = BOOTPO_END;
	do {
		if (secs == 4) {
			kprintf ("Giving up.\n");
			close (fd);
			return SYSERR;
		}
		pbp->bp_secs = hs2net(secs);
		secs++;
		kprintf ("Trying BOOTP request ...\n");
		if (write (fd, pug, pug->ug_len) == SYSERR) {
			kprintf ("bootp: error sending packet\n");
			close (fd);
			return SYSERR;
		}
	} while (recvtim (secs) == TIMEOUT);
	close (fd);
	return OK;
}

PROCESS bootp_in (int fd, int ppid, int xid, int ifnum)
{
	char buf[sizeof(struct bootp) + UG_HLEN];
	struct udpdgram *pug = (struct udpdgram *)buf;
	struct bootp *pbp = (struct bootp *)pug->ug_data;
	char addr[16];
	int i;

	while (1) {
		if (read (fd, buf, sizeof(buf)) == SYSERR) {
			kprintf ("Failed read.\n");
			send (ppid, SYSERR);
			return SYSERR;
		}
		if (pug->ug_port != UDPP_BOOTPS || pug->ug_nif != ifnum
		    || pbp->bp_xid != net2hl(xid))
			continue;
		/* Our reply */
		kprintf ("Processing reply ...\n");
		/* TODO check if we got all of packet */
		/* TODO vendor section may be more than 64 bytes */
		/* XXX */
		IPaddr mask = hl2net(0xffffff00);
		i = 4;
		do {
			if (i >= sizeof(pbp->bp_vend) - 1)
				break;
			if (pbp->bp_vend[i] != BOOTPO_PAD
			    && pbp->bp_vend[i] != BOOTPO_END) {
				if (i + 1 > sizeof(pbp->bp_vend) - 1
				    || i + pbp->bp_vend[i + 1] + 1
				       > sizeof(pbp->bp_vend) - 1)
					break;
			}
			switch (pbp->bp_vend[i]) {
			case BOOTPO_END:
				i = sizeof(pbp->bp_vend);
				break;
			case BOOTPO_PAD:
				i++;
				break;
			case BOOTPO_NETMASK:
				if (pbp->bp_vend[i + 1] != 4) {
					i = sizeof(pbp->bp_vend);
					break;
				}
				blkcopy(&mask, &pbp->bp_vend[i + 2], 4);
				i += 6;
				break;
			case BOOTPO_LPRSERVER:
				/* This is really the page server */
				if (pbp->bp_vend[i + 1] != 4) {
					i = sizeof(pbp->bp_vend);
					break;
				}
				blkcopy(&PSinfo.psip, &pbp->bp_vend[i + 2], 4);
				i += 6;
				break;
			default:
				i += 2 + pbp->bp_vend[i + 1];
			}
		} while (i < sizeof(pbp->bp_vend));

		ipif[ifnum].ipif_addr = pbp->bp_yiaddr;
		ipif[ifnum].ipif_mask = mask;
		ipif[ifnum].ipif_net = ipif[ifnum].ipif_addr
			               & ipif[ifnum].ipif_mask;
		ipif[ifnum].ipif_bcast = ipif[ifnum].ipif_net
			                 | ~ipif[ifnum].ipif_mask;
		iptochar (pbp->bp_yiaddr, addr);

		kprintf ("IP: %s\n", addr);
		send (ppid, OK);
		return OK;
	}
}
