#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <dns.h>
#include <stdio.h>

extern int ctr1000;

static void dnsmsg (struct dns *msg, u_short id, char *host);
static IPaddr dnsreadmsg (struct dns *msg, u_short id);
static PROCESS send_dnsreq (int dv, int ppid, u_short id, char *host);

/* Return IPADDR_ANY on error */
IPaddr hosttoip (char *host)
{
	int dv;
	u_short id;
	IPaddr addr;
	u_char buf[UG_HLEN + 512];
	struct dns *msg = (struct dns*)buf;

	if (host == NULL)
		return IPADDR_ANY;

        /* This should really be active */
	if ((dv = open (UDP, (int)"active:" DNS_SERVER, 0)) == SYSERR)
		return IPADDR_ANY;

	resume (create((int*) send_dnsreq, 1024, 40, "send_dnsreq", 4,
		dv, getpid (), id = ctr1000, host));
	
	while (1) {
		if (read (dv, buf, sizeof(buf)) == SYSERR)
			break;
		if ((addr = dnsreadmsg(msg, id)) != IPADDR_ANY) {
			close (dv);
			return addr;
		}
	}
	close (dv);
	return IPADDR_ANY;
}

static PROCESS send_dnsreq (int dv, int ppid, u_short id, char *host)
{
	int len = sizeof(struct dns) - 1 + strlen(host) + 1 + 4;
	u_char buf[len];
	struct dns *pdn = (struct dns*)buf;
	int i;

	dnsmsg (pdn, id, host);
	for (i = 0; i < DN_NTRIES; i++) {
		if (write (dv, buf, len) == SYSERR) {
			close (dv); /* receiver probably closed */
			return OK;
		}
		sleep (1);
	}
	close (dv);
	return OK;
}

/* perform dns lookup */
static void dnsmsg (struct dns *msg, u_short id, char *host)
{
	char *c;
	char *cur, *dot;

	msg->dn_id = hs2net(id);
	msg->dn_opparm = hs2net(0x100);
	msg->dn_qcount = hs2net(1);
	msg->dn_acount = 0;
	msg->dn_ncount = 0;
	msg->dn_dcount = 0;

	/* copy hostname in */
	cur = host;
	c = msg->dn_data;
	while (*cur != '\0') {
		int len;
		dot = strchr(cur, '.');
		len = (dot == NULL ? strlen (cur) : (dot - cur));
		*c = (u_char) len;
		strncpy (c + 1, cur, len);
		c += len + 1;
		if (*(cur + len) == '\0')
			break;
		cur += len + 1;
	}
	*(c++) = 0;
	*((u_short*)c) = net2hs(DN_QTHA);
	*((u_short*)(c+2)) = net2hs(DN_QTHA);
}

static IPaddr dnsreadmsg (struct dns *msg, u_short id)
{
	char *c;
	int i;

	if (id != net2hs(msg->dn_id) || !(net2hs(msg->dn_opparm) & 0x80)
	    || net2hs(msg->dn_opparm) & 0x7F)
		return IPADDR_ANY;
	c = msg->dn_data;

	/* question section */
	for (i = 0; i < net2hs(msg->dn_qcount); i++) {
		for (; *c != '\0'; c++);
		c += 5;
	}

	/* answer section */
	for (i = 0; i < net2hs(msg->dn_acount); i++) {
		//if (c > (u_char*)msg + 256)
			//return IPADDR_ANY;
		for (;*c != '\0'; c++) {
			if (*c & 0xC0) {
				c++;
				break;
			} else if (*c & 0x80 || *c & 0x40)
				return IPADDR_ANY;
		}
		c++;

		if (net2hs(*((u_short*)c)) == DN_QTHA
		    && net2hs(*((u_short*)(c+2))) == DN_QCIN) {
			u_short size;
			c += 8;
			size = net2hs(*((u_short*)c));
			if (size == 4)
				return *((u_int*)(c+2));
			c += size + 2;
		} else
			c += net2hs(*((u_short*)(c+8))) + 10;
	}
	return IPADDR_ANY;
}
