#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <timer.h>
#include <paging.h>
#include <page_server.h>

SYSCALL get_bs(bsd_t bs_id, unsigned int npages)
{
	int pagedev, ret, try = 0;
	struct ps_header req, resp;

	if (bs_id > MAX_ID || npages == 0
	    || (pagedev = get_pagedev()) == SYSERR)
		return SYSERR;
	bs_id += BASE_BS_ID;

	req.cmd = hl2net(PS_NEW);
	req.map_no = hl2net(bs_id);
	req.map_sz = hl2net(npages);
	req.seq = PSinfo.psseq++;

	do {
		write (pagedev, &req, PS_HEADER_SZ);
		tmset (PS_TIMEOUT, PSinfo.pstimerq, pagedev);
		if ((ret = read (pagedev, &resp, PS_HEADER_SZ)) == 0)
			continue;
		tmdel (PSinfo.pstimerq, pagedev);
		if (ret == SYSERR)
			return SYSERR;
		if (ret != PS_HEADER_SZ)
			continue;
		if (resp.seq == req.seq)
			break;
	} while (++try < MAX_PS_TRIES);
	
	if (try == MAX_PS_TRIES)
		return SYSERR;

	if (net2hl(resp.cmd) == PS_ACK)
		return net2hl(resp.map_sz);

	return SYSERR;
}
