#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <bufpool.h>
#include <timer.h>
#include <paging.h>
#include <page_server.h>

SYSCALL read_bs (char *dst, bsd_t bs_id, int page)
{
	int pagedev, ret, try = 0;
	struct ps_header req, *resp;

	if (bs_id > MAX_ID || (pagedev = get_pagedev ()) == SYSERR)
		return SYSERR;
	bs_id += BASE_BS_ID;

	resp = (struct ps_header *)getbuf (PSinfo.pspool);
	if (resp == (struct ps_header *)SYSERR)
		return SYSERR;

	req.cmd = hl2net (PS_GET);
	req.map_no = hl2net (bs_id);
	req.page_no = hl2net (page);
	req.seq = PSinfo.psseq++;

	do {
		write (pagedev, &req, PS_HEADER_SZ);
		tmset (PS_TIMEOUT, PSinfo.pstimerq, pagedev);
		if ((ret = read (pagedev, resp, PS_HEADER_SZ + NBPG)) == 0)
			continue;
		tmdel (PSinfo.pstimerq, pagedev);

		if (ret == SYSERR) {
			freebuf (resp);
			return SYSERR;
		}
		if (ret != PS_HEADER_SZ + NBPG)
			continue;
		if (resp->seq == req.seq)
			break;
	} while (++try < MAX_PS_TRIES);
	if (try == MAX_PS_TRIES) {
		freebuf (resp);
		return SYSERR;
	}

	if (net2hl (resp->cmd) == PS_PUT) {
		blkcopy (dst, (char *)resp + PS_HEADER_SZ, NBPG);
		freebuf (resp);
		return OK;
	}

	freebuf (resp);
	return SYSERR;
}
