#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <bufpool.h>
#include <timer.h>
#include <paging.h>
#include <page_server.h>

SYSCALL write_bs (char *src, bsd_t bs_id, int page)
{
	int pagedev, ret, try = 0;
	struct ps_header *req, resp;

	if (bs_id > MAX_ID || (pagedev = get_pagedev ()) == SYSERR)
		return SYSERR;
	bs_id += BASE_BS_ID;

	req = (struct ps_header *)getbuf (PSinfo.pspool);
	if (req == (struct ps_header *)SYSERR)
		return SYSERR;

	req->cmd = hl2net (PS_PUT);
	req->map_no = hl2net (bs_id);
	req->page_no = hl2net (page);
	req->seq = PSinfo.psseq++;
	blkcopy ((char *)req + PS_HEADER_SZ, src, NBPG);

	do {
		write (pagedev, req, PS_HEADER_SZ + NBPG);
		tmset (PS_TIMEOUT, PSinfo.pstimerq, pagedev);
		if ((ret = read (pagedev, &resp, PS_HEADER_SZ)) == 0)
			continue;
		tmdel (PSinfo.pstimerq, pagedev);
		if (ret == SYSERR) {
			freebuf (req);
			return SYSERR;
		}
		if (ret != PS_HEADER_SZ)
			continue;
		if (req->seq == resp.seq)
			break;
	} while (++try < MAX_PS_TRIES);

	freebuf (req);

	if (try == MAX_PS_TRIES)
		return SYSERR;

	if (net2hl (resp.cmd) == PS_ACK)
		return OK;

	return SYSERR;
}
