#include <conf.h>
#include <kernel.h>
#include <netif.h>
#include <ip.h>
#include <ipif.h>
#include <iproute.h>
#include <othernet.h>
#include <stdio.h>

COMMAND	x_if (int stdin, int stdout, int stderr, 
               int nargs, char *args[])
{
        int i;
        int ifnum;
        char buf[20];

        if(nargs==1) {
                fprintf(stdout, "Xinu interfaces\n");
                fprintf(stdout, "Bingid for this host = %d\n", bingid);
                fprintf(stdout, "No Stat  Ip               Mask             Net              Broadcast\n");
                for(i=0;i<NIF;i++)
                        if(nif[i].ni_state == NIS_UP) {
                                fprintf(stdout, "%02d UP    ", i);
                                fprintf(stdout, "%-15s  ", iptochar(ipif[i].ipif_addr, buf));
                                fprintf(stdout, "%-15s  ", iptochar(ipif[i].ipif_mask, buf));
                                fprintf(stdout, "%-15s  ", iptochar(ipif[i].ipif_net, buf));
                                fprintf(stdout, "%-15s  ", iptochar(ipif[i].ipif_bcast, buf));
                                fprintf(stdout, "\n");
                        }
                        else {
                                fprintf(stdout, "%02d DOWN  ", i);
                                fprintf(stdout, "\n");
                        }
                return(OK);
        }

        sscanf(args[1], "%d", &ifnum);

        if(nargs==2) {
                fprintf(stdout, "if: no command\n");
                return(OK);
        }

        if(strcmp(args[2], "up")==0) {
                if (ifnum == NI_OTHER1 || ifnum == NI_OTHER2)
                        ofaceinit (ifnum);
                nif[ifnum].ni_state = NIS_UP;
                return(OK);
        }

        if(strcmp(args[2], "down")==0) {
               	struct rttable *ri_working;
                struct route *prt;

		/* don't down the local interface */
                if (ifnum == NI_LOCAL) {
                        kprintf("if: cannot down local interface.\n");
                        return(SYSERR);
                }

                if (ifnum == NI_OTHER1 || ifnum == NI_OTHER2)
                        ofacerem (ifnum);
                nif[ifnum].ni_state = NIS_DOWN;

                wait (Route.ri_wsem);
                rtdelnb(ipif[ifnum].ipif_addr, RT_IPFULL); 
               	ri_working = Route.ri_working;

                for (prt=RT_PHEAD(ri_working); prt; prt=RT_PNEXT(ri_working, prt))
                        if(prt->rt_ifnum==ifnum)
                                rtdelnb(prt->rt_net, prt->rt_mask);

                signal (Route.ri_wsem);
                rtcommit();
                return(OK);
        }

        if(strcmp(args[2], "cfgip")==0) {
                if(nargs!=5) {
                        fprintf(stdout, "if: wrong cfg command\n");
                        return(-1);
                }
                ipif[ifnum].ipif_addr = chartoip(args[3]);
                ipif[ifnum].ipif_mask = chartoip(args[4]);
                ipif[ifnum].ipif_net  = ipif[ifnum].ipif_addr & ipif[ifnum].ipif_mask;
                ipif[ifnum].ipif_bcast = ipif[ifnum].ipif_net | (~ipif[ifnum].ipif_mask);
                return(OK);
        }

        if(strcmp(args[2], "cfgroute")==0) {
                int rt;
                if (nargs == 4)
                        sscanf(args[3], "%d", &rt);
                else
                        rt = 0;
                rtadd(ipif[ifnum].ipif_net, ipif[ifnum].ipif_mask, ipif[ifnum].ipif_addr, ifnum, RT_DFLT_METRIC, RT_TTLINF, rt, FALSE);
                rtadd(ipif[ifnum].ipif_addr, RT_IPFULL, ipif[ifnum].ipif_addr, NI_LOCAL, RT_DFLT_METRIC, RT_TTLINF, rt, FALSE);
                return(OK);
        }

        fprintf(stdout, "Error in interface command\n");
        return(-1);
}
