/* getbuf.c - getbuf */

#include <conf.h>
#include <kernel.h>
#include <mark.h>
#include <bufpool.h>
#include <stdio.h>

/*------------------------------------------------------------------------
 *  getbuf  --  get a buffer from a preestablished buffer pool
 *------------------------------------------------------------------------
 */
int *getbuf(int poolid)
{
	STATWORD ps;    
	int	*buf, inuse;

#ifdef	MEMMARK
	if ( unmarked(bpmark) )
		return((int *) SYSERR);
#endif
	if (poolid<0 || poolid>=nbpools)
		return((int *) SYSERR);
	wait(bptab[poolid].bpsem);
	disable(ps);
	inuse = bptab[poolid].bptotal - scount(bptab[poolid].bpsem);
	if (inuse > bptab[poolid].bpmaxused)
		bptab[poolid].bpmaxused = inuse;
	buf = (int *) bptab[poolid].bpnext;
	bptab[poolid].bpnext = (char *) *buf;
	*buf++ = poolid;
	restore(ps);
	return( (int *) buf );
}

/*------------------------------------------------------------------------
 * nbgetbuf - a non-blocking version of getbuf
 *------------------------------------------------------------------------
 */
int *nbgetbuf(int poolid)
{
	STATWORD ps;    
	int	*buf;
	int	inuse;

#ifdef	MEMMARK
	if ( unmarked(bpmark) )
		return((int *) SYSERR);
#endif
	if (poolid<0 || poolid>=nbpools)
		return((int *) SYSERR);
	disable(ps);
	if (scount(bptab[poolid].bpsem) <= 0) {
		restore(ps);
		return 0;
	}
	wait(bptab[poolid].bpsem);	/* shouldn't block */
	inuse = bptab[poolid].bptotal - scount(bptab[poolid].bpsem);
	if (inuse > bptab[poolid].bpmaxused)
		bptab[poolid].bpmaxused = inuse;
	buf = (int *) bptab[poolid].bpnext;
	bptab[poolid].bpnext = (char *) *buf;
	*buf++ = poolid;
	restore(ps);
	return( (int *) buf );
}
