#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ipif.h>
#include <dns.h>
#include <iproute.h>
#include <stdio.h>
#include <mq.h>

#define UDP_NTRIES	32

static int isportfree (IPaddr addr, u_short port)
{
	int i;
	for (i = 0; i < Nudp; i++) {
		if (ucbtab[i].ucb_state == UCB_OPEN
		    && ucbtab[i].ucb_lip == addr
		    && ucbtab[i].ucb_lport == port) {
			return SYSERR;
		}
	}
	return OK;
}

int udpmopen (struct devsw *pdev, void *arg2, void *arg3)
{
	IPaddr ip, lip;
	u_short port, lport;
	int active, i, udpdev;
	struct route *prt;

	if (cspecparse ((char *)arg2, &ip, &port, &active) == SYSERR)
		return SYSERR;

	if (active) {
		if (port == UDPP_ANY || ip == IPADDR_ANY)
			return SYSERR;

		prt = rtget(ip);
		if (prt == NULL)
			return SYSERR;
		lip = ipif[prt->rt_ifnum].ipif_addr;
		rtfree (prt);

		wait (Udp.devmutex);
		for (i = 0; i < UDP_NTRIES; i++) {
			lport = (rand() & 0xfff) + 1024;
			if (isportfree (lip, hs2net (lport)))
				break;
		}
		if (i == UDP_NTRIES) {
			signal (Udp.devmutex);
			return SYSERR;
		}

		/* XXX: We should probably just keep track of what ports
		 * are taken. */
		for (i = 0; i < Nudp; i++)
			if (ucbtab[i].ucb_state == UCB_FREE)
				break;
		if (i == Nudp) {
			signal (Udp.devmutex);
			return SYSERR;
		}

		udpdev = i;
		ucbtab[udpdev].ucb_state = UCB_OPEN;
		ucbtab[udpdev].ucb_flags = UCBF_DEFAULT;
		ucbtab[udpdev].ucb_lip = lip;
		ucbtab[udpdev].ucb_rip = ip;
		ucbtab[udpdev].ucb_lport = hs2net (lport);
		ucbtab[udpdev].ucb_rport = hs2net (port);
	} else {
		wait (Udp.devmutex);
		if (port == UDPP_ANY) {
			for (i = 0; i < UDP_NTRIES; i++) {
				port = (rand() & 0xfff) + 1024;
				if (isportfree (ip, hs2net (port)) == OK)
					break;
			}
			if (i == UDP_NTRIES) {
				signal (Udp.devmutex);
				return SYSERR;
			}
		} else {
			if (isportfree (ip, hs2net (port)) == SYSERR) {
				signal (Udp.devmutex);
				return SYSERR;
			}
		}

		for (i = 0; i < Nudp; i++)
			if (ucbtab[i].ucb_state == UCB_FREE)
				break;
		if (i == Nudp) {
			signal (Udp.devmutex);
			return SYSERR;
		}

		udpdev = i;
		ucbtab[udpdev].ucb_state = UCB_OPEN;
		ucbtab[udpdev].ucb_flags = UCBF_DEFAULT;
		ucbtab[udpdev].ucb_lip = ip;
		ucbtab[udpdev].ucb_rip = IPADDR_ANY;
		ucbtab[udpdev].ucb_lport = hs2net (port);
		ucbtab[udpdev].ucb_rport = UDPP_ANY;
	}

	mqenable (ucbtab[i].ucb_mq);
	signal (Udp.devmutex);
	return ucbtab[udpdev].ucb_dvnum;
}
