#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <mq.h>
#include <stdio.h>

int udpread (struct devsw *pdev, char *buf, int len)
{
	STATWORD ps;
	struct ucb *pucb = &ucbtab[pdev->dvminor];
        struct udpdgram *pug;
	struct ep *pep;
	struct ip *pip;
	struct udp *pudp;
        int connected = 0, readlen, cookie;

	wait (Udp.devmutex);
	if (pucb->ucb_state != UCB_OPEN) {
		signal (Udp.devmutex);
		return SYSERR;
	}
        if (pucb->ucb_rip != IPADDR_ANY)
            connected = 1;
	disable(ps);
	cookie = pucb->ucb_cookie;
	signal (Udp.devmutex);

	if (cookie != pucb->ucb_cookie) {
		restore(ps);
		return SYSERR;
	}

	if ((pep = (struct ep *)mqrecv (pucb->ucb_mq)) == (struct ep *)SYSERR) {
		restore(ps);
		return SYSERR;
	}
	restore(ps);
	if (pep == NULL)
		return 0;
	pip = (struct ip *)pep->ep_data;
	pudp = (struct udp *)((char *)pip + IP_HLEN(pip));

        if (connected) {
		readlen = min (net2hs (pudp->udp_len) - UDP_HLEN, len);
		blkcopy (buf, pudp->udp_data, readlen);
        } else {
		readlen = min (net2hs (pudp->udp_len - UDP_HLEN),
			       len - UG_HLEN);
		pug = (struct udpdgram *)buf;
		pug->ug_ip = pip->ip_src;
		pug->ug_port = net2hs (pudp->udp_src);
		pug->ug_len = net2hs (pudp->udp_len) - UDP_HLEN;
		pug->ug_nif = pep->ep_inif;
		blkcopy (pug->ug_data, pudp->udp_data, readlen);
	}

	freebuf (pep);

	return readlen;
}
