#include <conf.h>
#include <kernel.h>
#include <network.h>
#include <ipif.h>
#include <stdio.h>

int udpwrite (struct devsw *pdev, char *buf, int len)
{
	struct ucb *pucb = &ucbtab[pdev->dvminor];
	struct ep *pep;
	struct ip *pip;
	struct udp *pudp;
	struct udpdgram *pug = (struct udpdgram *)buf;
	IPaddr lip, rip;
	int connected = 0, flags;
	u_short lport, rport;
	u_char ttl;

	wait (Udp.devmutex);
	if (pucb->ucb_state != UCB_OPEN) {
		signal (Udp.devmutex);
		return SYSERR;
	}
	/* These prevent some locking in inconvenient places */
	lip = pucb->ucb_lip;
	rip = pucb->ucb_rip;
	flags = pucb->ucb_flags;
	lport = pucb->ucb_lport;
	rport = pucb->ucb_rport;
	ttl = pucb->ucb_ttl;
	if (pucb->ucb_rip != IPADDR_ANY)
		connected = 1;
	signal (Udp.devmutex);

	if (connected) {
		pep = ipalloc (Udp.udpdev, len + UDP_HLEN, lip, rip);
	} else {
		pep = ipalloc (Udp.udpdev, len + UDP_HLEN, lip,
			       (pug->ug_ip == IPADDR_BCAST || IP_ISMULTI(pug->ug_ip))
			       ? IPADDR_ANY : pug->ug_ip);
	}
	if (pep == NULL)
		return SYSERR;

	pip = (struct ip *)pep->ep_data;
	pudp = (struct udp *)((char *)pip + IP_HLEN(pip));

	if (connected) {
		pip->ip_src = lip;
		pip->ip_dst = rip;
		pudp->udp_src = lport;
		pudp->udp_dst = rport;
	} else {
		/* Broadcast is so ugly */
		if (pug->ug_ip == IPADDR_BCAST || IP_ISMULTI(pug->ug_ip)) {
			pep->ep_outif = pug->ug_nif;
			pip->ip_src = ipif[pug->ug_nif].ipif_addr;
			pip->ip_dst = pug->ug_ip;
		}
		pudp->udp_src = lport;
		pudp->udp_dst = hs2net (pug->ug_port);
	}
	if (flags & UCBF_TTL)
		pip->ip_ttl = ttl;

	if (connected) {
		pudp->udp_len = hs2net (len + UDP_HLEN);
		blkcopy (pudp->udp_data, buf, len);
	} else {
		pudp->udp_len = hs2net (len + UDP_HLEN);
		blkcopy (pudp->udp_data, pug->ug_data, len);
	}

	pudp->udp_cksum = 0;
	if (flags & UCBF_CKSUM)
		pudp->udp_cksum = udpcksum (pip, pudp);

	ipsend (pep);

	return OK;
}
